#ifndef PYTHIAPLUGIN_h
#define PYTHIAPLUGIN_h
/*********************************************/
/* INTERFACE CLASSES TO Pythia8              */
/* feed pythia with a completely new process */
/*********************************************/
//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;
using namespace Pythia8;

namespace Moses1 {


// A derived class for a completely new, user defined (from scratch) process
class FGB : public Sigma2Process {
	public:
		// Constructor.
		FGB(int idIn, int nKKmodes, string pdfSet, int pdfMember) :
				m_idNew(idIn), m_nKKmodes(nKKmodes), m_pdfSet(pdfSet), m_pdfMember(pdfMember) {}

		// initialize the process:
		void initProc();

		// Calculate flavour-independent parts of cross section.
		virtual void sigmaKin();

		// Evaluate sigmaHat(sHat).
		virtual double sigmaHat();

		// Select flavour, colour and anticolour.
		virtual void setIdColAcol();


		virtual string	name()		const {return "qqbar-> G(KK) ->t tbar";}
		virtual int	code()		const {return 100003;}
		virtual string	inFlux()	const {return "ffbarSame";}
		virtual bool    hasIntermediate() const {return true;}
		virtual int     idIntermediate()  const {return 5000021;}
		virtual int	resonanceA()	const {return 5000021;}
		virtual bool	isSChannel()	const {return true;}
		virtual int	id3Mass()	const {return m_idNew;}
		virtual int	id4Mass()	const {return m_idNew;}
		virtual bool	convert2mb()	const {return true;}


	private:
		// Store flavour-specific process information and standard prefactor.
		double m_sigma;
		int m_idNew;
                bool m_isPhysical;
                customParticle m_outFermion1, m_inFermion1, m_outFermion2, m_inFermion2;
                dcomplex m_fullAmp;
                double m_totalwidth_Z0, m_totalwidth_gammaKK, m_totalwidth_ZKK;
                double m_totalwidth_GKK[3][9];
                double m_mr, m_betaf, m_cosThe;
                double m_thetaDistribution;
                double m_openFracPair;
		int m_nKKmodes;

		parameters m_prm; // create an instance of the parameters ***class***
		physicalParameters m_physPrm; // create an instance of the physical parameters ***struct***
		runParameters m_runPrm; // create an instance of the run parameters ***struct***
		PDTparticle m_pdtPrt; // set the class for PDTparticle call
		CUSTOMparticle m_customPrt;  // set the class for CUSTOMparticle call
		couplings m_cplng; // set the class for couplings call
		partialWidth m_wdt; // set the class for partialWidth call
		helicityAmplitude m_hAmp; // set the class for helicityAmplitude call
		rapidity m_rap;  // set the class for rapidity call
		lhapdf m_lha; // set the class for lhapdf call
		string m_pdfSet;
		int m_pdfMember;

		// set a vector to contain all pdtParticle structs of properties
		vector<pdtParticle> m_PDTfermions;
		vector<pdtParticle> m_PDTbosons;
		vector<customParticle> m_CUSTOMfermions; // define a vector of custom particles
};


}
#endif
