/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------
//#include "analyzer.h"

using namespace std;
using namespace Moses1;
using namespace Pythia8;
//using namespace RooFit;

int main(int argc, char* argv[]) {
	cout << "Holy Moses..." << endl;

	// set styles
	TStyle* s = (TStyle*)gROOT->GetStyle("Plain");
	s->cd();
	gStyle->SetOptStat(11);

	// declare the application
	TApplication theApp("App", &argc, argv);

	commonOperations commonOps; // set the class for commonOperations call
	// set styles
	commonOps.SetOKStyle();

	// set the prefix
	string dirName = "/";
	string outFileName = dirName + "Resolution_kinematics";
	cout << "\n" << endl;


	// declare the relevant classes
	cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
	cutsEmulator cuts; // create the cuts emulator instance (input to rapidity)
	dileptonAnalyzer anlz;
	bool doTrue = true;
	anlz.setTrue(doTrue);
	Particle particle;

	double minM = 200;
	double maxM = 1000;
	cout << "choose mHat min...";
	cin >> minM;
	cout << "choose mHat max...";
        cin >> maxM;
	

	// set cuts
	//double invMassCut=1000.;
	double invMassCut=minM;
	double pTransverseCut=10.;
	double EtaCut=2.5;
	int cutSelector = 1;
	cout << "select cuts to apply: no cuts (-1) all cuts (0), only mHat cut (1), only pT cut (2), only eta cut (3)..." << endl;
	cin >> cutSelector;
	switch(cutSelector) {
		case -1: pTransverseCut = 0.;
			 EtaCut = 1e10;
			 invMassCut = 0.;
			 break;
		case  0: break;
		case  1: pTransverseCut = 0.;
			 EtaCut = 1e10;
			 break;
		case  2: invMassCut = 0.;
			 EtaCut = 1e10;
			 break;
		case  3: invMassCut = 0.;
			 pTransverseCut = 0.;
			 break;
		default: cout << "wrong option cutSelector = " << cutSelector << "\t set to default cutSelector=0" << endl;
			 break;
	}
	anlz.setCuts(invMassCut, pTransverseCut, EtaCut);


	int sampleSizeSelector = 1;
	cout << "select input sample size: 100k(1), L=500fb^-1(2), L=100fb^-1(3), L=10fb^-1(4)..." << endl;
	cin >> sampleSizeSelector;
	if (sampleSizeSelector<1 || sampleSizeSelector>4) {
		cout << "invalid option, set to 1 (default)" << endl;
		sampleSizeSelector = 1;
	}
	string sSampleSize = "";
	switch (sampleSizeSelector) {
		case 1: sSampleSize = "_LrefinvFb"; break;
		case 2: sSampleSize = "_L500invFb"; break;
		case 3: sSampleSize = "_L100invFb"; break;
		case 4: sSampleSize = "_L10invFb"; break;
		default: break;
	}
	outFileName += sSampleSize;

	
	int mStarSelector = 4;
	string sMStarSelector = "_mStar=4TeV";
	if(sampleSizeSelector==4) {
		cout << "select m^* value: 3TeV(3), 4TeV(4)...";
		cin >> mStarSelector;
		if (mStarSelector==3) {
			sMStarSelector = "_mStar=3TeV";
			physPrm.mStar = 3000.;
                        physPrm.mZprime = 3000.;
                        physPrm.wZprime = 80.;
		}
		else {
			sMStarSelector = "_mStar=4TeV";
			physPrm.mStar = 4000.;
                        physPrm.mZprime = 4000.;
                        physPrm.wZprime = 120.;
		}
	}
	outFileName += sMStarSelector;


	string resolutionRootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + ".root");
	string resolutionEpsFileName  = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + ".eps");

	TFile *resolutionHistFile = new TFile(resolutionRootFileName.c_str(),"recreate");


	TDirectory* dir_allHistograms = resolutionHistFile->mkdir("allHistograms");
        dir_allHistograms->cd();

        TDirectory* dir_resolution = resolutionHistFile->mkdir("resolution");
        dir_resolution->cd();
	TCanvas *canv_resolution = new TCanvas("canv_resolution","canv_resolution",0,0,1400,800);
	canv_resolution->Divide(2,3);

	TVirtualPad *pad_mHat=canv_resolution->cd(1);
        pad_mHat->SetLogy();
	TVirtualPad *pad_pT=canv_resolution->cd(2);
        pad_pT->SetLogy();
	TVirtualPad *pad_mHatRes=canv_resolution->cd(3);
	TVirtualPad *pad_pTRes=canv_resolution->cd(4);
	TVirtualPad *pad_2dmHatRes=canv_resolution->cd(5);
        TVirtualPad *pad_2dpTRes=canv_resolution->cd(6);
	

	TDirectory* dir_mHat_true = resolutionHistFile->mkdir("mHat_true");
        dir_mHat_true->cd();
	TCanvas *canv_mHat_true = new TCanvas("canv_mHat_true","canv_mHat_true",0,0,1400,800);
        canv_mHat_true->SetLogy();

	TDirectory* dir_mHat_smeared = resolutionHistFile->mkdir("mHat_smeared");
        dir_mHat_smeared->cd();
        TCanvas *canv_mHat_smeared = new TCanvas("canv_mHat_smeared","canv_mHat_smeared",0,0,1400,800);
        canv_mHat_smeared->SetLogy();

	TDirectory* dir_mHat_resolution = dir_resolution->mkdir("mHat_resolution");
        dir_mHat_resolution->cd();
        TCanvas *canv_mHat_resolution = new TCanvas("canv_mHat_resolution","canv_mHat_resolution",0,0,1400,800);

	TDirectory* dir_mHat_resolution2d = dir_resolution->mkdir("mHat_resolution2d");
        dir_mHat_resolution2d->cd();
        TCanvas *canv_mHat_resolution2d = new TCanvas("canv_mHat_resolution2d","canv_mHat_resolution2d",0,0,1400,800);



	TDirectory* dir_pT_true = resolutionHistFile->mkdir("pT_true");
        dir_pT_true->cd();
        TCanvas *canv_pT_true = new TCanvas("canv_pT_true","canv_pT_true",0,0,1400,800);
        canv_pT_true->SetLogy();
        
        TDirectory* dir_pT_smeared = resolutionHistFile->mkdir("pT_smeared");
        dir_pT_smeared->cd();
        TCanvas *canv_pT_smeared = new TCanvas("canv_pT_smeared","canv_pT_smeared",0,0,1400,800);
        canv_pT_smeared->SetLogy();

	TDirectory* dir_pT_resolution = dir_resolution->mkdir("pT_resolution");
        dir_pT_resolution->cd();
        TCanvas *canv_pT_resolution = new TCanvas("canv_pT_resolution","canv_pT_resolution",0,0,1400,800);

        TDirectory* dir_pT_resolution2d = dir_resolution->mkdir("pT_resolution2d");
        dir_pT_resolution2d->cd();
        TCanvas *canv_pT_resolution2d = new TCanvas("canv_pT_resolution2d","canv_pT_resolution2d",0,0,1400,800);
	


	// boundries and bins of histograms
	Int_t invMnBins = 50;
	Double_t invMmin = invMassCut, invMmax = maxM;

	// choose mHat & cosTheta binnings
	switch(sampleSizeSelector) {
		case 1:	invMnBins = 50; break; // 1M sample
		case 2: invMnBins = 14; break; // 500[fb^-1] sample
		case 3: invMnBins = 14; break; // 100[fb^-1] sample
		case 4: invMnBins = 19; break; // 10 fb^-1 sample
		default: break;
	}


	Int_t    pTnBins = 50; // 100
	Double_t pTmin   = 10.;
	Double_t pTmax   = 4000.;
	Double_t pTBoundToReduce = 0.;//1000.;
	Double_t invMassBoundToReduce = 0.;//1000.;
	if (sampleSizeSelector==2 || sampleSizeSelector==3) { // small samples
		pTnBins = 20;
	}
	else if (sampleSizeSelector==4)
	{
		pTmax =  1200;
		pTBoundToReduce = 0;
		pTnBins = 14;
		invMnBins = 19;
		invMassBoundToReduce = 0.;
	}


	if (cutSelector==-1 || cutSelector==1 || cutSelector==2 || cutSelector==3) {
		pTnBins = 100;
		pTmin = 5.;
		pTmax = 100.;
		
		invMnBins = 500;
		invMmin = 5.;
		invMmax = 6000.;
	}


	// logarithmic boundries and bins of histograms
	Double_t logMmin = log10(invMmin);
	Double_t logMmax = log10(invMmax - invMassBoundToReduce);
	Double_t M_binwidth = (Double_t)( (logMmax-logMmin)/(Double_t)invMnBins );
	Double_t M_bins[invMnBins+1];
	M_bins[0] = invMmin;
	for (Int_t i=1 ; i<=invMnBins ; i++) { M_bins[i] = TMath::Power( 10,(logMmin + i*M_binwidth) ); }

	Double_t logpTmin = log10(pTmin);
	Double_t logpTmax = log10(pTmax - pTBoundToReduce);
	Double_t pT_binwidth = (Double_t)( (logpTmax-logpTmin)/(Double_t)pTnBins );
	Double_t pT_bins[pTnBins+1];
	pT_bins[0] = pTmin;
	for (Int_t i=1 ; i<=pTnBins ; i++) { pT_bins[i] = TMath::Power( 10,(logpTmin + i*pT_binwidth) ); }


	
	//----- SM -----------------------------------------------------------------------------------------------------
	dir_mHat_smeared->cd();
	TH1 *SM_mHat = new TH1D("SM_mHat", "", invMnBins, invMmin, invMmax);
	SM_mHat->SetLineStyle(5);
	SM_mHat->SetLineColor(kBlue);
	SM_mHat->SetLineWidth(2);
	SM_mHat->SetXTitle("#hat{m} [GeV]");
	SM_mHat->SetYTitle("Events");

	dir_mHat_true->cd();
	TH1 *SM_mHatTrue = new TH1D("SM_mHatTrue", "", invMnBins, invMmin, invMmax);
        SM_mHatTrue->SetLineStyle(5);
        SM_mHatTrue->SetLineColor(kBlue);
        SM_mHatTrue->SetXTitle("#hat{m}[GeV]");
        SM_mHatTrue->SetYTitle("Events");

	dir_pT_smeared->cd();
	TH1 *SM_pT = new TH1D("SM_pT", "", pTnBins, pTmin, pTmax);
	SM_pT->SetLineStyle(5);
	SM_pT->SetLineColor(kBlue);
        SM_pT->SetLineWidth(2);
	SM_pT->SetXTitle("p_{T} [GeV]");
	SM_pT->SetYTitle("Events");

	dir_pT_true->cd();
	TH1 *SM_pTTrue = new TH1D("SM_pTTrue", "", pTnBins, pTmin, pTmax);
        SM_pTTrue->SetLineStyle(5);
        SM_pTTrue->SetXTitle("p_{T} [GeV]");
        SM_pTTrue->SetYTitle("Events");



	//----- KK -----------------------------------------------------------------------------------------------------
	dir_mHat_smeared->cd();
	TH1 *KK_mHat = new TH1D("KK_mHat", "", invMnBins, invMmin, invMmax);
	KK_mHat->SetLineStyle(1);
	KK_mHat->SetLineColor(kRed);
	KK_mHat->SetLineWidth(2);
	KK_mHat->SetXTitle("#hat{m} [GeV]");
	KK_mHat->SetYTitle("Events");
	
	dir_mHat_true->cd();
	TH1 *KK_mHatTrue = new TH1D("KK_mHatTrue", "", invMnBins, invMmin, invMmax);
        KK_mHatTrue->SetLineStyle(1);
        KK_mHatTrue->SetLineColor(kRed);
        KK_mHatTrue->SetXTitle("#hat{m} [GeV]");
        KK_mHatTrue->SetYTitle("Events");

	dir_pT_smeared->cd();
	TH1 *KK_pT = new TH1D("KK_pT", "", pTnBins, pTmin, pTmax);
	KK_pT->SetLineStyle(1);
	KK_pT->SetLineColor(kRed);
        KK_pT->SetLineWidth(2);
	KK_pT->SetXTitle("p_{T} [GeV]");
	KK_pT->SetYTitle("Events");

	dir_pT_true->cd();
	TH1 *KK_pTTrue = new TH1D("KK_pTTrue", "", pTnBins, pTmin, pTmax);
        KK_pTTrue->SetLineStyle(1);
	KK_pTTrue->SetLineColor(kRed);
        KK_pTTrue->SetXTitle("p_{T} [GeV]");
        KK_pTTrue->SetYTitle("Events");

	
	//----- Z' -----------------------------------------------------------------------------------------------------
	dir_mHat_smeared->cd();
	TH1 *Zprime_mHat = new TH1D("Zprime_mHat", "", invMnBins, invMmin, invMmax);
	Zprime_mHat->SetLineStyle(3);
        Zprime_mHat->SetLineColor(kBlack);
	Zprime_mHat->SetLineWidth(2);
	Zprime_mHat->SetXTitle("#hat{m} [GeV]");
	Zprime_mHat->SetYTitle("Events");

	dir_mHat_true->cd();
	TH1 *Zprime_mHatTrue = new TH1D("Zprime_mHatTrue", "", invMnBins, invMmin, invMmax);
        Zprime_mHatTrue->SetLineStyle(3);
	Zprime_mHatTrue->SetLineColor(kBlack);
        Zprime_mHatTrue->SetXTitle("#hat{m} [GeV]");
        Zprime_mHatTrue->SetYTitle("Events");

	dir_pT_smeared->cd();
	TH1 *Zprime_pT = new TH1D("Zprime_pT", "", pTnBins, pTmin, pTmax);
	Zprime_pT->SetLineStyle(3);
	Zprime_pT->SetLineColor(kBlack);
        Zprime_pT->SetLineWidth(2);
	Zprime_pT->SetXTitle("p_{T} [GeV]");
	Zprime_pT->SetYTitle("Events");

	dir_pT_true->cd();
	TH1 *Zprime_pTTrue = new TH1D("Zprime_pTTrue", "", pTnBins, pTmin, pTmax);
        Zprime_pTTrue->SetLineStyle(3);
	Zprime_pTTrue->SetLineColor(kBlack);
        Zprime_pTTrue->SetXTitle("p_{T} [GeV]");
        Zprime_pTTrue->SetYTitle("Events");




	// resolution invariant mass histos 
	Double_t resMin=-1., resMax=+1.;
	Int_t resnBins = 100;
	if (sampleSizeSelector == 4) { // 10[fb^-1]
		resMin=-0.2;
		resMax=+0.2;
	}

	dir_mHat_resolution->cd();
        TH1D *SM_mHat_resolution = new TH1D("SM_mHat_resolution", "", resnBins, resMin, resMax);
        SM_mHat_resolution->SetLineStyle(5);
        SM_mHat_resolution->SetLineColor(kBlue);
        SM_mHat_resolution->SetXTitle("#left(#hat m^{rec}-#hat m^{tru}#right)/#hat{m}^{tru}");
        SM_mHat_resolution->SetYTitle("Events");

	TH1D *KK_mHat_resolution = new TH1D("KK_mHat_resolution", "", resnBins, resMin, resMax);
	KK_mHat_resolution->SetLineStyle(1);
        KK_mHat_resolution->SetLineColor(kRed);
        KK_mHat_resolution->SetXTitle("#left(#hat m^{rec}-#hat m^{tru}#right)/#hat{m}^{tru}");
        KK_mHat_resolution->SetYTitle("Events");

        TH1D *Zprime_mHat_resolution = new TH1D("Zprime_mHat_resolution", "", resnBins, resMin, resMax);
	Zprime_mHat_resolution->SetLineStyle(3);
        Zprime_mHat_resolution->SetXTitle("#left(#hat m^{rec}-#hat m^{tru}#right)/#hat{m}^{tru}");
        Zprime_mHat_resolution->SetYTitle("Events");

	dir_pT_resolution->cd();
        TH1D *SM_pT_resolution = new TH1D("SM_pT_resolution", "", resnBins, resMin, resMax);
	SM_pT_resolution->SetLineStyle(5);
        SM_pT_resolution->SetLineColor(kBlue);
        SM_pT_resolution->SetXTitle("#left(1/p_{T}^{rec}-1/p_{T}^{tru}#right)/#left(1/p_{T}^{tru}#right)");
        SM_pT_resolution->SetYTitle("Events");

        TH1D *KK_pT_resolution = new TH1D("KK_pT_resolution", "", resnBins, resMin, resMax);
	KK_pT_resolution->SetLineStyle(1);
        KK_pT_resolution->SetLineColor(kRed);
        KK_pT_resolution->SetXTitle("#left(1/p_{T}^{rec}-1/p_{T}^{tru}#right)/#left(1/p_{T}^{tru}#right)");
        KK_pT_resolution->SetYTitle("Events");

        TH1D *Zprime_pT_resolution = new TH1D("Zprime_pT_resolution", "", resnBins, resMin, resMax);
	Zprime_pT_resolution->SetLineStyle(3);
        Zprime_pT_resolution->SetXTitle("#left(1/p_{T}^{rec}-1/p_{T}^{tru}#right)/#left(1/p_{T}^{tru}#right)");
        Zprime_pT_resolution->SetYTitle("Events");




	// ------- 2d ---------------------------------------------------------------------------------------------------------
	dir_mHat_resolution2d->cd();
	//TH2 *SM2d_mHat_resolution = new TH2D("SM2d_mHat_resolution", "", invMnBins, invMmin, invMmax, resnBins, resMin, resMax);
	TH2 *SM2d_mHat_resolution = new TH2D("SM2d_mHat_resolution", "", invMnBins, M_bins, resnBins, resMin, resMax);
	SM2d_mHat_resolution->SetLineColor(kBlue);
	//SM2d_mHat_resolution->SetLineStyle(5);
	SM2d_mHat_resolution->SetFillColor(kBlue);
        SM2d_mHat_resolution->SetXTitle("#hat{m}^{tru} [GeV]");
        SM2d_mHat_resolution->SetYTitle("#hat{m} resolution");
        SM2d_mHat_resolution->SetZTitle("Events");

	//TH2 *KK2d_mHat_resolution = new TH2D("KK2d_mHat_resolution", "", invMnBins, invMmin, invMmax, resnBins, resMin, resMax);
	TH2 *KK2d_mHat_resolution = new TH2D("KK2d_mHat_resolution", "", invMnBins, M_bins, resnBins, resMin, resMax);
        KK2d_mHat_resolution->SetLineColor(kRed);
	//KK2d_mHat_resolution->SetLineStyle(1);
	KK2d_mHat_resolution->SetFillColor(kRed);
        KK2d_mHat_resolution->SetXTitle("#hat{m}^{tru} [GeV]");
        KK2d_mHat_resolution->SetYTitle("#hat{m} resolution");
        KK2d_mHat_resolution->SetZTitle("Events");

	//TH2 *Zprime2d_mHat_resolution = new TH2D("Zprime2d_mHat_resolution", "", invMnBins, invMmin, invMmax, resnBins, resMin, resMax);
	TH2 *Zprime2d_mHat_resolution = new TH2D("Zprime2d_mHat_resolution", "", invMnBins, M_bins, resnBins, resMin, resMax);
        Zprime2d_mHat_resolution->SetLineColor(kBlack);
	//Zprime2d_mHat_resolution->SetLineStyle(3);
	Zprime2d_mHat_resolution->SetFillColor(kBlack);
        Zprime2d_mHat_resolution->SetXTitle("#hat{m}^{tru} [GeV]");
        Zprime2d_mHat_resolution->SetYTitle("#hat{m} resolution");
        Zprime2d_mHat_resolution->SetZTitle("Events");
	


	dir_pT_resolution2d->cd();
	//TH2 *SM2d_pT_resolution = new TH2D("SM2d_pT_resolution", "", invMnBins, invMmin, invMmax, resnBins, resMin, resMax);
	TH2 *SM2d_pT_resolution = new TH2D("SM2d_pT_resolution", "", pTnBins, pT_bins, resnBins, resMin, resMax);
        SM2d_pT_resolution->SetLineColor(kBlue);
	//SM2d_pT_resolution->SetLineStyle(5);
	SM2d_pT_resolution->SetFillColor(kBlue);
        SM2d_pT_resolution->SetXTitle("p_{T}^{tru} [GeV]");
        SM2d_pT_resolution->SetYTitle("p_{T} resolution");
        SM2d_pT_resolution->SetZTitle("Events");

        //TH2 *KK2d_pT_resolution = new TH2D("KK2d_pT_resolution", "", invMnBins, invMmin, invMmax, resnBins, resMin, resMax);
	TH2 *KK2d_pT_resolution = new TH2D("KK2d_pT_resolution", "", pTnBins, pT_bins, resnBins, resMin, resMax);
        KK2d_pT_resolution->SetLineColor(kRed);
	//KK2d_pT_resolution->SetLineStyle(1);
	KK2d_pT_resolution->SetFillColor(kRed);
        KK2d_pT_resolution->SetXTitle("p_{T}^{tru} [GeV]");
        KK2d_pT_resolution->SetYTitle("p_{T} resolution");
        KK2d_pT_resolution->SetZTitle("Events");

        //TH2 *Zprime2d_pT_resolution = new TH2D("Zprime2d_pT_resolution", "", invMnBins, invMmin, invMmax, resnBins, resMin, resMax);
	TH2 *Zprime2d_pT_resolution = new TH2D("Zprime2d_pT_resolution", "", pTnBins, pT_bins, resnBins, resMin, resMax);
        Zprime2d_pT_resolution->SetLineColor(kBlack);
        //prime2d_pT_resolution->SetLineStyle(3);
	Zprime2d_pT_resolution->SetFillColor(kBlack);
	Zprime2d_pT_resolution->SetXTitle("p_{T}^{tru} [GeV]");
        Zprime2d_pT_resolution->SetYTitle("p_{T} resolution");
        Zprime2d_pT_resolution->SetZTitle("Events");


	


	// ------ legends -----------------------------------------------------------------------------------------------------
	TLegend *leg_mHat_smeared = new TLegend(0.67,0.65,0.9,0.85);
        leg_mHat_smeared->AddEntry(SM_mHat,"SM^{rec}","L");
        leg_mHat_smeared->AddEntry(KK_mHat,"KK^{rec}","L");
        leg_mHat_smeared->AddEntry(Zprime_mHat,"Z'_{SM}^{rec}","L");	

	TLegend *leg_mHat_true = new TLegend(0.67,0.65,0.9,0.85);
	leg_mHat_true->AddEntry(SM_mHatTrue,"SM^{tru}","L");
        leg_mHat_true->AddEntry(KK_mHatTrue,"KK^{tru}","L");
        leg_mHat_true->AddEntry(Zprime_mHatTrue,"Z'_{SM}^{tru}","L");


	TLegend *leg_mHat = new TLegend(0.73,0.55,0.96,0.9);
        leg_mHat->AddEntry(SM_mHat,"SM^{rec}","L");
        leg_mHat->AddEntry(KK_mHat,"KK^{rec}","L");
        leg_mHat->AddEntry(Zprime_mHat,"Z'_{SM}^{rec}","L");
	leg_mHat->AddEntry(SM_mHatTrue,"SM^{tru}","L");
        leg_mHat->AddEntry(KK_mHatTrue,"KK^{tru}","L");
        leg_mHat->AddEntry(Zprime_mHatTrue,"Z'_{SM}^{tru}","L");
	

	TLegend *leg_pT_smeared = new TLegend(0.67,0.65,0.9,0.85);
        leg_pT_smeared->AddEntry(SM_pT,"SM^{rec}","L");
        leg_pT_smeared->AddEntry(KK_pT,"KK^{rec}","L");
        leg_pT_smeared->AddEntry(Zprime_pT,"Z'_{SM}^{rec}","L");

        TLegend *leg_pT_true = new TLegend(0.67,0.65,0.9,0.85);
        leg_pT_true->AddEntry(SM_pTTrue,"SM^{tru}","L");
        leg_pT_true->AddEntry(KK_pTTrue,"KK^{tru}","L");
        leg_pT_true->AddEntry(Zprime_pTTrue,"Z'_{SM}^{tru}","L");
	

	TLegend *leg_pT = new TLegend(0.73,0.55,0.96,0.9);
        leg_pT->AddEntry(SM_pT,"SM^{rec}","L");
        leg_pT->AddEntry(KK_pT,"KK^{rec}","L");
        leg_pT->AddEntry(Zprime_pT,"Z'_{SM}^{rec}","L");
	leg_pT->AddEntry(SM_pTTrue,"SM^{tru}","L");
        leg_pT->AddEntry(KK_pTTrue,"KK^{tru}","L");
        leg_pT->AddEntry(Zprime_pTTrue,"Z'_{SM}^{tru}","L");

	TLegend *leg_mHatRes = new TLegend(0.7,0.5,0.96,0.7);
	leg_mHatRes->AddEntry(SM_mHat_resolution,"SM","L");
	leg_mHatRes->AddEntry(KK_mHat_resolution,"KK","L");
	leg_mHatRes->AddEntry(Zprime_mHat_resolution,"Z'_{SM}","L");

	TLegend *leg_pTRes = new TLegend(0.7,0.5,0.96,0.7);
        leg_pTRes->AddEntry(SM_pT_resolution,"SM","L");
        leg_pTRes->AddEntry(KK_pT_resolution,"KK","L");
        leg_pTRes->AddEntry(Zprime_pT_resolution,"Z'_{SM}","L");
	
	TLegend *leg_2dmHatRes = new TLegend(0.17,0.17,0.43,0.37);
	leg_2dmHatRes->AddEntry(SM2d_mHat_resolution,"SM","F");
	leg_2dmHatRes->AddEntry(KK2d_mHat_resolution,"KK","F");
	leg_2dmHatRes->AddEntry(Zprime2d_mHat_resolution,"Z'_{SM}","F");

	TLegend *leg_2dpTRes = new TLegend(0.17,0.17,0.43,0.37);
        leg_2dpTRes->AddEntry(SM2d_pT_resolution,"SM","F");
        leg_2dpTRes->AddEntry(KK2d_pT_resolution,"KK","F");
        leg_2dpTRes->AddEntry(Zprime2d_pT_resolution,"Z'_{SM}","F");


	//----------------------------------------------------------------------------------------------------------
	
	string filePath;
	int Nevents;
	vector<fullRecord> allgoodLeptons, allgoodaLeptons, allgoodLeptonsTrue, allgoodaLeptonsTrue;
	int Nleptons, Naleptons, NleptonsTrue, NaleptonsTrue;
	double invariantMass, pTransverse, invariantMassTrue, pTransverseTrue;
	double pTRes, mHatRes;

	// to count the number of events in the KK peak area
	double KKpeak_highLimit = 5000.;
	double KKpeak_lowLimit = 2000.;
	if (sampleSizeSelector==4) {
		KKpeak_highLimit = 2000.;
		KKpeak_lowLimit = 200.;
	}
	

	//string treeType = "testTree";
	string treeType = "testTree_allPairs";

	// loop over 2 files (with / without Initial State Radiation)
	for (int f=1 ; f<=3 ; f++) {
		//-------------------------------------------------------------------------------------------------------------------
		// read the trees and put all good lepton pairs into 2 vectors
		//----- 1M
		if (sampleSizeSelector == 1) { //1M smaples
			if (f==1) { // SM  //KK_smeared_extractedParticles_2trees_100000_EventsSample
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"100k/SM_smeared_extractedParticles_2trees_100000_EventsSample.root");
			}
			else if (f==2) { // KK
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"100k/KK_smeared_extractedParticles_2trees_100000_EventsSample.root");
			}
			else if (f==3) { // E6
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"100k/E6_smeared_extractedParticles_2trees_100000_EventsSample.root");
			}
		}

		//----- 500
		else if (sampleSizeSelector == 2) { // 500[fb^-1]
			if (f==1) { // SM
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"500inversefemtobarn/"+"SM_extractedParticles_2trees_500inversefemtobarnSample.root");
			}
                        else if (f==2) { // KK
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"500inversefemtobarn/"+"KK_extractedParticles_2trees_500inversefemtobarnSample.root");
                        }
                        else if (f==3) { // E6
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"500inversefemtobarn/"+"E6_extractedParticles_2trees_500inversefemtobarnSample.root");
			}
		}

		//----- 100
		else if (sampleSizeSelector == 3) { // 100[fb^-1]
			if (f==1) { // SM
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"SM_smeared_extractedParticles_2trees_100inversefemtobarnSample.root");
			}
			else if (f==2) { // KK
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_smeared_extractedParticles_2trees_100inversefemtobarnSample.root");
			}
			else if (f==3) { // E6
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"E6_smeared_extractedParticles_2trees_100inversefemtobarnSample.root");
			}
		}
		else if (sampleSizeSelector == 4) { // 10[fb^-1]
		 	if (f==1) { // SM
				if (mStarSelector==3) {
		 			filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"SM_smeared_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar4TeV.root");
				}
				else if(mStarSelector==4) {
					filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"SM_smeared_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar4TeV.root");
				}
			}
			else if (f==2) { // KK
				if (mStarSelector==3) {
					filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_smeared_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar3TeV.root");
				}
				else if(mStarSelector==4) {
					filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_smeared_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar4TeV.root");
				}
			}
			else if (f==3) { // E6
				if (mStarSelector==3) {
					filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"E6_smeared_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar3TeV.root");
				}
				else if(mStarSelector==4) {
					filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"E6_smeared_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar4TeV.root");
				}
			}
		}




		Nevents = anlz.loopOverEvents(filePath, treeType.c_str());
		cout << "Nevents = " << Nevents << endl;
		// get all the good pairs (after cuts)
		Nleptons      = anlz.getAllLeptons(allgoodLeptons);
		Naleptons     = anlz.getAllaLeptons(allgoodaLeptons);
		NleptonsTrue  = anlz.getAllLeptonsTrue(allgoodLeptonsTrue);
                NaleptonsTrue = anlz.getAllaLeptonsTrue(allgoodaLeptonsTrue);
		cout << "Nleptons = " << Nleptons << endl;
		cout << "NleptonsTrue = " << NleptonsTrue << endl;
		cout << "NgoodLeptons = " << anlz.allGoodLeptons() << endl;
		if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}
		cout << "\n" << endl;
		//-------------------------------------------------------------------------------------------------------------------
	
		// run over the good pairs and fill the histograms
		for (int k=0 ; k<Nleptons ; k++) {
			invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
			pTransverse = allgoodLeptons[k].pT;
			invariantMassTrue = anlz.invMass(allgoodLeptonsTrue[k], allgoodaLeptonsTrue[k]);
                        pTransverseTrue = allgoodLeptonsTrue[k].pT;
			mHatRes = (invariantMass-invariantMassTrue)/invariantMassTrue;
			pTRes   = (1./pTransverse-1./pTransverseTrue)/(1./pTransverseTrue);
			// fill the histos
			if (f==1) {
				SM_mHat->Fill( (Double_t)invariantMass );
				SM_pT->Fill( (Double_t)pTransverse );
				SM_mHatTrue->Fill( (Double_t)invariantMassTrue );
                                SM_pTTrue->Fill( (Double_t)pTransverseTrue );
				SM_mHat_resolution->Fill( mHatRes );
				SM_pT_resolution->Fill( pTRes );
				SM2d_mHat_resolution->Fill( invariantMassTrue, mHatRes );
				SM2d_pT_resolution->Fill( pTransverseTrue, pTRes );
			}
			else if (f==2) {
				KK_mHat->Fill( (Double_t)invariantMass );
                                KK_pT->Fill( (Double_t)pTransverse );
                                KK_mHatTrue->Fill( (Double_t)invariantMassTrue );
                                KK_pTTrue->Fill( (Double_t)pTransverseTrue );
                                KK_mHat_resolution->Fill( mHatRes );
                                KK_pT_resolution->Fill( pTRes );
				KK2d_mHat_resolution->Fill( invariantMassTrue, mHatRes );
                                KK2d_pT_resolution->Fill( pTransverseTrue, pTRes );
			}
			else if (f==3) {
				Zprime_mHat->Fill( (Double_t)invariantMass );
                                Zprime_pT->Fill( (Double_t)pTransverse );
                                Zprime_mHatTrue->Fill( (Double_t)invariantMassTrue );
                                Zprime_pTTrue->Fill( (Double_t)pTransverseTrue );
                                Zprime_mHat_resolution->Fill( mHatRes );
                                Zprime_pT_resolution->Fill( pTRes );
				Zprime2d_mHat_resolution->Fill( invariantMassTrue, mHatRes );
                                Zprime2d_pT_resolution->Fill( pTransverseTrue, pTRes );
			}
		}
		// clear the pairs vectors
		anlz.clearAllLeptons();
		if (allgoodLeptons.size() != 0) { allgoodLeptons.clear(); }
		if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }
		if (allgoodLeptonsTrue.size() != 0) { allgoodLeptonsTrue.clear(); }
                if (allgoodaLeptonsTrue.size() != 0) { allgoodaLeptonsTrue.clear(); }
	}


	cout << "\n" << endl;


	if (sampleSizeSelector==1) {

		SM_mHat->Scale(1.2);
		SM_pT->Scale(1.2);

		Zprime_mHat->Scale(1.15);
		Zprime_pT->Scale(1.15);
	}

	double max=0., max1=0., max2=0., max3=0.;

	dir_resolution->cd();
	canv_resolution->Draw();
	canv_resolution->cd();
	
	pad_mHat->cd();
        SM_mHat->Draw();
        Zprime_mHat->Draw("sames");
        KK_mHat->Draw("sames");
	SM_mHatTrue->Draw("sames");
        Zprime_mHatTrue->Draw("sames");
        KK_mHatTrue->Draw("sames");
        leg_mHat->Draw("sames");

	pad_pT->cd();
        SM_pT->Draw();
        Zprime_pT->Draw("sames");
        KK_pT->Draw("sames");
        SM_pTTrue->Draw("sames");
        Zprime_pTTrue->Draw("sames");
        KK_pTTrue->Draw("sames");
        leg_pT->Draw("sames");

	max1 = SM_mHat_resolution->GetMaximum();
        max2 = KK_mHat_resolution->GetMaximum();
	max3 = Zprime_mHat_resolution->GetMaximum();
        max = (max1 > max2) ? max1 : max2;
	max = (max3 > max) ? max3 : max;
        SM_mHat_resolution->SetMaximum(1.05*max);

	pad_mHatRes->cd();
	SM_mHat_resolution->Draw();
	Zprime_mHat_resolution->Draw("sames");
	KK_mHat_resolution->Draw("sames");
	leg_mHatRes->Draw("sames");

	max1 = SM_pT_resolution->GetMaximum();
	max2 = KK_pT_resolution->GetMaximum();
        max3 = Zprime_pT_resolution->GetMaximum();
        max = (max1 > max2) ? max1 : max2;
	max = (max3 > max) ? max3 : max; 
        SM_pT_resolution->SetMaximum(1.05*max);

        pad_pTRes->cd();
	SM_pT_resolution->Draw();
        Zprime_pT_resolution->Draw("sames");
        KK_pT_resolution->Draw("sames");
        leg_pTRes->Draw("sames");

        pad_2dmHatRes->cd();
	SM2d_mHat_resolution->Draw("BOX");
        Zprime2d_mHat_resolution->Draw("boxsames");
        KK2d_mHat_resolution->Draw("boxsames");
        leg_2dmHatRes->Draw("sames");	

	pad_2dpTRes->cd();
        SM2d_pT_resolution->Draw("BOX");
        Zprime2d_pT_resolution->Draw("boxsames");
        KK2d_pT_resolution->Draw("boxsames");
        leg_2dpTRes->Draw("sames");

	canv_resolution->Update();
	canv_resolution->Write();


	
	dir_mHat_true->cd();
        canv_mHat_true->Draw();
        canv_mHat_true->cd();
	SM_mHatTrue->Draw();
        Zprime_mHatTrue->Draw("sames");
        KK_mHatTrue->Draw("sames");
	leg_mHat_true->Draw("sames");
	canv_mHat_true->Update();
        canv_mHat_true->Write();
	
	dir_mHat_smeared->cd();
        canv_mHat_smeared->Draw();
        canv_mHat_smeared->cd();
        SM_mHat->Draw();
        Zprime_mHat->Draw("sames");
        KK_mHat->Draw("sames");
	leg_mHat_smeared->Draw("sames");
        canv_mHat_smeared->Update();
        canv_mHat_smeared->Write();


	dir_pT_true->cd();
        canv_pT_true->Draw();
        canv_pT_true->cd();
        SM_pTTrue->Draw();
        Zprime_pTTrue->Draw("sames");
        KK_pTTrue->Draw("sames");
	leg_pT_true->Draw("sames");
        canv_pT_true->Update();
        canv_pT_true->Write();

        dir_pT_smeared->cd();
        canv_pT_smeared->Draw();
        canv_pT_smeared->cd();
        SM_pT->Draw();
        Zprime_pT->Draw("sames");
        KK_pT->Draw("sames");
	leg_pT_smeared->Draw("sames");
        canv_pT_smeared->Update();
        canv_pT_smeared->Write();


	
	dir_mHat_resolution->cd();
        canv_mHat_resolution->Draw();
        canv_mHat_resolution->cd();
        SM_mHat_resolution->Draw();
        Zprime_mHat_resolution->Draw("sames");
        KK_mHat_resolution->Draw("sames");
	leg_mHatRes->Draw("sames");
        canv_mHat_resolution->Update();
        canv_mHat_resolution->Write();

	dir_mHat_resolution2d->cd();
        canv_mHat_resolution2d->Draw();
        canv_mHat_resolution2d->cd();
        SM2d_mHat_resolution->Draw("box");
        Zprime2d_mHat_resolution->Draw("boxsames");
        KK2d_mHat_resolution->Draw("boxsames");
	leg_2dmHatRes->Draw("sames");
        canv_mHat_resolution2d->Update();
        canv_mHat_resolution2d->Write();



	dir_pT_resolution->cd();
        canv_pT_resolution->Draw();
        canv_pT_resolution->cd();
        SM_pT_resolution->Draw();
        Zprime_pT_resolution->Draw("sames");
        KK_pT_resolution->Draw("sames");
	leg_pTRes->Draw("sames");
        canv_pT_resolution->Update();
        canv_pT_resolution->Write();

        dir_pT_resolution2d->cd();
        canv_pT_resolution2d->Draw();
        canv_pT_resolution2d->cd();
        SM2d_pT_resolution->Draw("box");
        Zprime2d_pT_resolution->Draw("boxsames");
        KK2d_pT_resolution->Draw("boxsames");
	leg_2dpTRes->Draw("sames");
        canv_pT_resolution2d->Update();
        canv_pT_resolution2d->Write();

	

	cout << "\n---finalizing---\n" << endl;
	
	resolutionHistFile->Write();

	canv_resolution->Print(resolutionEpsFileName.c_str());


	// wait so the plots can hold on
	canv_resolution->WaitPrimitive();

	// Done.
	return 0;
}

