/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;
using namespace Moses1;
using namespace Pythia8;


int main(int argc, char* argv[]) {
	cout << "Holy Moses..." << endl;

	// set styles
	TStyle* s = (TStyle*)gROOT->GetStyle("Plain");
	s->cd();
	gStyle->SetOptStat(11);

	commonOperations commonOps; // set the class for commonOperations call
	// set styles
	commonOps.SetOKStyle();

	// set the prefix
	string dirName = "/KKvsZE6vsSMstudy/";

	// set the reference sample model
	int iRefModel = 1;
	string sRefModel = "KK";
	cout << "enter a reference model KK(1), SM(2), E6(3) or GR(4) ?" << endl;
	cout << "..." << endl;
	cin >> iRefModel;
	switch(iRefModel) {
		case 1: sRefModel = "KK"; break;
		case 2: sRefModel = "SM"; break;
		case 3: sRefModel = "E6"; break;
		case 4: sRefModel = "GR"; break;
		default: cout << "wrong option, set to default: KK(1)" << endl;
			 sRefModel = "KK"; break;
			 break;
	}
	string sGravitonSampleSize = "";
	if (iRefModel==4) {
		cout << "select graviton reference sample size 10k / 50k / 100k / 1M ... " << endl;
		cin >> sGravitonSampleSize;
		if (sGravitonSampleSize!="10k" && sGravitonSampleSize!="50k" && sGravitonSampleSize!="100k" && sGravitonSampleSize!="1M") {
			cout << "invalid option, set to 100k (default)" << endl;
			sGravitonSampleSize = "100k";
		}
	}

	// set the reference sample size
	int refSampleSizeSelector = 1;
	if(iRefModel==4) {refSampleSizeSelector = 1;}
	else {
		cout << "select reference sample size: 1M(1), L=500fb^-1(2), L=100fb^-1(3)..." << endl;
		cin >> refSampleSizeSelector;
	}
	if (refSampleSizeSelector!=1 && refSampleSizeSelector!=2 && refSampleSizeSelector!=3) {
		cout << "invalid option, set to 1 (default)" << endl;
		refSampleSizeSelector = 1;
	}

	string sRefSampleSize = "";
	switch (refSampleSizeSelector) {
		case 1: sRefSampleSize = "Lreference"; break;
		case 2: sRefSampleSize = "L500invFb"; break;
		case 3: sRefSampleSize = "L100invFb"; break;
		default: break;
	}


	// set the input sample size
	int inputSampleSizeSelector = 1;
	cout << "select input sample size: 1M(1), L=500fb^-1(2), L=100fb^-1(3), L=10fb^-1(4)..." << endl;
	cin >> inputSampleSizeSelector;
	if (inputSampleSizeSelector!=1 && inputSampleSizeSelector!=2 && inputSampleSizeSelector!=3 && inputSampleSizeSelector!=4) {
		cout << "invalid option, set to 1 (default)" << endl;
		inputSampleSizeSelector = 1;
	}
	string sInputSampleSize = "";
	switch (inputSampleSizeSelector) {
		case 1: sInputSampleSize = "Lreference"; break;
		case 2: sInputSampleSize = "L500invFb"; break;
		case 3: sInputSampleSize = "L100invFb"; break;
		case 4: sInputSampleSize = "L10invFb"; break;
		default: break;
	}

/*
	string sInputMstar = "mStar4TeV";
	int mStarSelector = 4;
	if (inputSampleSizeSelector==4) {
		cout << "choose m^* value: 3TeV(3), 4TeV(4)...";
		cin >> mStarSelector;
		if (mStarSelector!=3 && mStarSelector!=4) {
			cout << "invalid option, set to 4 (default)" << endl;
			mStarSelector = 4;
		}
		switch(mStarSelector) {
			case 3: sInputMstar = "mStar3TeV"; break;
			case 4: sInputMstar = "mStar4TeV"; break;
			default: break;
		}
	}
*/

	// choose reference frame
	int referenceFrameSelector = 0;
	string cosThetaTitle;
	cout << "choose reference frame to calculate cos(theta)" << endl;
	cout << "(1) Helicity frame or (2) Collins-Soper frame" << endl;
	cout << "..." << endl;
	cin >> referenceFrameSelector;
	if (referenceFrameSelector != 1 && referenceFrameSelector != 2) {
		cout << "invalid option, setting to default = 1" << endl;
		referenceFrameSelector = 1;
	}
	if (referenceFrameSelector == 1) {cosThetaTitle = "Helicity";}
	else if (referenceFrameSelector == 2) {cosThetaTitle = "Collins-Soper";}


	int sampleSelector = 1;
	string withORwithoutCuts;
	cout << "select sample type: withCuts(1), noCuts(2)..." << endl;
	cin >> sampleSelector;
	if (sampleSelector!=1 && sampleSelector!=2) {
		cout << "wrong option: sampleSelector=" << sampleSelector << " setting to default (1)" << endl;
		sampleSelector = 1;
	}
	else {
		switch(sampleSelector) {
			case 1: withORwithoutCuts = "_withCuts"; break;
			case 2: withORwithoutCuts = "_noCuts"; break;
			default: break;
		}
	}


	int distributionSelector = 1;
	cout << "choose the issue distribution to run the K test on" << endl;
        cout << "mHat(1), cosTheta(2), pT(3) ... ";
        cin >> distributionSelector;
	string yTitle = "";
	string distName = "";
	double min   = 0.;
	double max   = 0.;
	if (distributionSelector < 1 || distributionSelector > 3) {
		cout << "invalid option, setting to default = 1" << endl;
		distributionSelector = 1;
	}
	else {
		switch(distributionSelector) {
			case 1: // mHat
				yTitle = "Kolmogorov test #left(for #frac{d#sigma}{d#hat{m}}#right)";
				distName = "_mHat";
				min = 1000.;
				max = 6000.;
				break;
			case 2: // cosTheta
				yTitle = "Kolmogorov test #left(for #frac{d#sigma}{dcos#theta '}#right)";
				distName = "_cosTheta";
				min = -1.;
				max = +1.;
				break;
			case 3: // pT
				yTitle = "Kolmogorov test #left(for #frac{d#sigma}{dp_{T}}#right)";
				distName = "_pT";
				min = 10.;
				max = 2500.;
				break;
		}
	}


	//--- select static method of calculating costheta
	int quarkDirectionOrRapidity = 2; // 1=quark direction, 2=rapidity sign

	TApplication theApp("App", &argc, argv);

	bool doRun = false;
	int runSelector = 0;
        cout << "choose to calculate at a given binning(0) or draw graphs with many binnings(1)..." << endl;
        cin >> runSelector;
	if (runSelector==1) { doRun = true; }
	else if (runSelector==0) { cout << "not running !" << endl; }
	else { cout << "wrong option, setting to default (not running)" << endl; runSelector=0; }

	int errorSelector = 0;
	if (runSelector==0) {
		cout << "choose to calculate at less bins(1), or at ,more bins(2)..." << endl;
		cin >> errorSelector;
		if (errorSelector!=1 && errorSelector!=2) {
			cout << "invalid option, setting to default = 0" << endl;
			errorSelector = 0;
		}
	}

	// set the out file name prefix
	string outFileName = dirName + "Kolmogorov/" + "KolmogorovTest";




	// write the results toi tree:

	string outTreeFilePath = "";
	TFile *outTreeFile = NULL;
	string outTreeName = "";
	TTree *outTree = NULL;
	
	Int_t    numberOfBins = 0;
	Double_t SM_kolmogorovTest = 0.;
	Double_t E6_kolmogorovTest = 0.;
	Double_t KK_kolmogorovTest = 0.;

	if (doRun) {
		outTreeFilePath = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_allModels_" + sInputSampleSize + "_vs_" + sRefModel + "_" + sRefSampleSize + withORwithoutCuts + distName + "_TREE.root");
		outTreeFile = new TFile(outTreeFilePath.c_str(),"recreate");
		outTreeName = "KolmogorovTest_" + sRefModel + "_" + sRefSampleSize + distName;
		outTree = new TTree(outTreeName.c_str(),outTreeName.c_str());

	        outTree->Branch("numberOfBins",&numberOfBins,"numberOfBins/I");
		outTree->Branch("SM_kolmogorovTest",&SM_kolmogorovTest,"SM_kolmogorovTest/D");
		outTree->Branch("E6_kolmogorovTest",&E6_kolmogorovTest,"E6_kolmogorovTest/D");
		outTree->Branch("KK_kolmogorovTest",&KK_kolmogorovTest,"KK_kolmogorovTest/D");
	}

	// declare the relevant classes
	cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
	cutsEmulator cuts; // create the cuts emulator instance (input to rapidity)
	dileptonAnalyzer anlz;
	Particle particle;

	// set cuts
	double invMassCut=1000.;
	double pTransverseCut=10.;
	double EtaCut=2.5;

	double no_invMassCut=0.;
	double no_pTransverseCut=0.;
	double no_EtaCut=1e10;



	TFile *Kolmogorov_HistFile = NULL;
	TCanvas *canv = NULL;
	string Kolmogorov_RootFileName = "";
	string Kolmogorov_EpsFileName = "";
	if (doRun) {
		// declare kinematics histograms and root items
		Kolmogorov_RootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_allModels_" + sInputSampleSize + "_vs_" + sRefModel + "_" + sRefSampleSize + withORwithoutCuts + distName + ".root");
		Kolmogorov_EpsFileName  = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_allModels_" + sInputSampleSize + "_vs_" + sRefModel + "_" + sRefSampleSize + withORwithoutCuts + distName + ".eps");


		Kolmogorov_HistFile = new TFile(Kolmogorov_RootFileName.c_str(),"recreate");
		canv = new TCanvas("canv","Kaluza-Klein Kolmogorov test",0,0,1400,800);
		canv->SetLogx();
	}

	string filePath;
	int Nevents_ref, Nevents_refBG, Nevents_input_KK, Nevents_input_E6, Nevents_input_SM;
	vector<fullRecord> allRefGoodLeptons, allRefGoodaLeptons, allRefBGGoodLeptons, allRefBGGoodaLeptons,KK_allInputGoodLeptons, KK_allInputGoodaLeptons, E6_allInputGoodLeptons, E6_allInputGoodaLeptons, SM_allInputGoodLeptons, SM_allInputGoodaLeptons;
	int Nleptons_ref, Naleptons_ref, Nleptons_refBG=0, Naleptons_refBG=0, Nleptons_input_KK, Naleptons_input_KK, Nleptons_input_E6, Naleptons_input_E6, Nleptons_input_SM, Naleptons_input_SM;
	double cosThetaBoost, cosThetaCollinsSoper;
	double invariantMass, pT, cosTh=0.;

	// to check the asymmetry in a narrow range arrownd the peak
	double mZ_known = physPrm.mStar;
	double mZ_lowLimit = physPrm.mStar-2000.;
	double mZ_highLimit = physPrm.mStar+1000.;
	cout << "\n" << "mZ_known=" << mZ_known << "  mZ_lowLimit=" << mZ_lowLimit << "  mZ_highLimit=" << mZ_highLimit << "\n" << endl;

	
	//---------------------- read the tree and fill the histos
	// set the cuts and select the tree
	string treeName;
	if (sampleSelector==1) {
		//treeName = "testTree";
		treeName = "testTree_allPairs";
		anlz.setCuts(invMassCut,pTransverseCut,EtaCut);
	}
	else if (sampleSelector==2) {
		treeName = "testTree_allPairs";
		anlz.setCuts(no_invMassCut,no_pTransverseCut,no_EtaCut);
	}



	// read the reference terees and put all good lepton pairs into 2 vectors
	if (refSampleSizeSelector==1) { // sample size = 1M
		if (iRefModel==4 && sGravitonSampleSize!="1M") {
			filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/"+"GR_extractedParticles_2trees_"+sGravitonSampleSize+"_EventsSample.root");
		}
		else {
			filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/"+sRefModel+"_extractedParticles_2trees_1Msample.root");
		}
	}
	else if (refSampleSizeSelector==2) { // L=500fb^-1
		filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"500inversefemtobarn/"+sRefModel+"_extractedParticles_2trees_500inversefemtobarnSample.root");
	}
	else if (refSampleSizeSelector==3) { // L=100fb^-1
		filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"100inversefemtobarn/"+sRefModel+"_extractedParticles_2trees_100inversefemtobarnSample.root");
	}
	// read the tree
	Nevents_ref = anlz.loopOverEvents(filePath,treeName.c_str());
	cout << "Nevents_ref = " << Nevents_ref << endl;
	// get all the good pairs (after whatever cuts applied before) and put them in vectors
	Nleptons_ref = anlz.getAllLeptons(allRefGoodLeptons);
	Naleptons_ref = anlz.getAllaLeptons(allRefGoodaLeptons);
	cout << "Nleptons_ref = " << Nleptons_ref << endl;
	if (Nleptons_ref != Naleptons_ref) {cout << "warning: Nleptons_ref  != Naleptons_ref , exitting" << endl;    exit(-1);}
	cout << "\n" << endl;

	if(iRefModel==4) {
		// read the SM background of the G signal
		filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/SMforGR_extractedParticles_2trees_1Msample.root");
		Nevents_refBG = anlz.loopOverEvents(filePath,treeName.c_str());
		cout << "Nevents_refBG = " << Nevents_refBG << endl;
		// get all the good pairs (after whatever cuts applied before) and put them in vectors
		Nleptons_refBG = anlz.getAllLeptons(allRefBGGoodLeptons);
		Naleptons_refBG = anlz.getAllaLeptons(allRefBGGoodaLeptons);
		cout << "Nleptons_refBG = " << Nleptons_refBG << endl;
		if (Nleptons_refBG != Naleptons_refBG) {cout << "warning: Nleptons_refBG != Naleptons_refBG , exitting" << endl;    exit(-1);}
		cout << "\n" << endl;
	}

	
	//-------------------------------------------------------------------------------------------------------------------
	string sInputModel = "KK";
	// read the input terees and put all good lepton pairs into 2 vectors
	if (inputSampleSizeSelector==1) { // sample size = 1M
		filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/"+sInputModel+"_extractedParticles_2trees_1Msample"+".root");
	}
	else if (inputSampleSizeSelector==2) { // L=500fb^-1
		filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"500inversefemtobarn/"+sInputModel+"_extractedParticles_2trees_500inversefemtobarnSample.root");
	}
	else if (inputSampleSizeSelector==3) { // L=100fb^-1
		filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"100inversefemtobarn/"+sInputModel+"_extractedParticles_2trees_100inversefemtobarnSample.root");
	}
	// read the tree
	Nevents_input_KK = anlz.loopOverEvents(filePath,treeName.c_str());
	cout << "Nevents_input_KK = " << Nevents_input_KK << endl;
	// get all the good pairs (after whatever cuts applied before) and put them in vectors
	Nleptons_input_KK  = anlz.getAllLeptons(KK_allInputGoodLeptons);
	Naleptons_input_KK = anlz.getAllaLeptons(KK_allInputGoodaLeptons);
	cout << "Nleptons_input_KK = " << Nleptons_input_KK << endl;
	if (Nleptons_input_KK != Naleptons_input_KK) {cout << "warning: Nleptons_input_KK != Naleptons_input_KK, exitting" << endl;    exit(-1);}
	cout << "\n" << endl;
	//-------------------------------------------------------------------------------------------------------------------
        sInputModel = "E6";
        // read the input terees and put all good lepton pairs into 2 vectors
        if (inputSampleSizeSelector==1) { // sample size = 1M
                filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/"+sInputModel+"_extractedParticles_2trees_1Msample"+".root");
        }
        else if (inputSampleSizeSelector==2) { // L=500fb^-1
                filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"500inversefemtobarn/"+sInputModel+"_extractedParticles_2trees_500inversefemtobarnSample.root");
        }
        else if (inputSampleSizeSelector==3) { // L=100fb^-1
                filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"100inversefemtobarn/"+sInputModel+"_extractedParticles_2trees_100inversefemtobarnSample.root");
        }
        // read the tree
        Nevents_input_E6 = anlz.loopOverEvents(filePath,treeName.c_str());
        cout << "Nevents_input_E6 = " << Nevents_input_E6 << endl;
        // get all the good pairs (after whatever cuts applied before) and put them in vectors
        Nleptons_input_E6  = anlz.getAllLeptons(E6_allInputGoodLeptons);
        Naleptons_input_E6 = anlz.getAllaLeptons(E6_allInputGoodaLeptons);
        cout << "Nleptons_input_E6 = " << Nleptons_input_E6 << endl;
        if (Nleptons_input_E6 != Naleptons_input_E6) {cout << "warning: Nleptons_input_E6 != Naleptons_input_E6, exitting" << endl;    exit(-1);}
        cout << "\n" << endl;
	//-------------------------------------------------------------------------------------------------------------------
        sInputModel = "SM";
        // read the input terees and put all good lepton pairs into 2 vectors
        if (inputSampleSizeSelector==1) { // sample size = 1M
                filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/"+sInputModel+"_extractedParticles_2trees_1Msample"+".root");
        }
        else if (inputSampleSizeSelector==2) { // L=500fb^-1
                filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"500inversefemtobarn/"+sInputModel+"_extractedParticles_2trees_500inversefemtobarnSample.root");
        }
        else if (inputSampleSizeSelector==3) { // L=100fb^-1
                filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"100inversefemtobarn/"+sInputModel+"_extractedParticles_2trees_100inversefemtobarnSample.root");
        }
        // read the tree
        Nevents_input_SM = anlz.loopOverEvents(filePath,treeName.c_str());
        cout << "Nevents_input_SM = " << Nevents_input_SM << endl;
        // get all the good pairs (after whatever cuts applied before) and put them in vectors
        Nleptons_input_SM  = anlz.getAllLeptons(SM_allInputGoodLeptons);
        Naleptons_input_SM = anlz.getAllaLeptons(SM_allInputGoodaLeptons);
        cout << "Nleptons_input_SM = " << Nleptons_input_SM << endl;
        if (Nleptons_input_SM != Naleptons_input_SM) {cout << "warning: Nleptons_input_SM != Naleptons_input_SM, exitting" << endl;    exit(-1);}
        cout << "\n" << endl;
	//-------------------------------------------------------------------------------------------------------------------



	// make the unbinned test:
	vector<Double_t> vec_ref_mHat; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	vector<Double_t> vec_ref_cosTheta; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	for (int k=0 ; k<Nleptons_ref ; k++) {
		invariantMass = anlz.invMass(allRefGoodLeptons[k], allRefGoodaLeptons[k]);
		cosThetaBoost = anlz.cosThetaBoost(allRefGoodLeptons[k], allRefGoodaLeptons[k]);
		cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allRefGoodLeptons[k], allRefGoodaLeptons[k]);
		if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
		else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
		// fill the vectors
		vec_ref_mHat.push_back(invariantMass);
		if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
			vec_ref_cosTheta.push_back(cosTh);
		}
	}
	if (iRefModel==4) {
		for (int k=0 ; k<Nleptons_refBG ; k++) {
			invariantMass = anlz.invMass(allRefBGGoodLeptons[k], allRefBGGoodaLeptons[k]);
			cosThetaBoost = anlz.cosThetaBoost(allRefBGGoodLeptons[k], allRefBGGoodaLeptons[k]);
			cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allRefBGGoodLeptons[k], allRefBGGoodaLeptons[k]);
			if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
			else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
			// fill the vectors
			vec_ref_mHat.push_back(invariantMass);
			if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
				vec_ref_cosTheta.push_back(cosTh);
			}
		}

	}
	cout << "vec_ref_mHat.size() = " << vec_ref_mHat.size() << endl;
	cout << "vec_ref_cosTheta.size() = " << vec_ref_cosTheta.size() << endl;
	
	// clear the pairs vectors
	anlz.clearAllLeptons();
	Int_t refEntries_mHat = (Int_t)vec_ref_mHat.size();
	Int_t refEntries_cosTheta = (Int_t)vec_ref_cosTheta.size();
	Double_t *array_ref_mHat = (Double_t *)malloc(refEntries_mHat*sizeof(Double_t));
	Double_t *array_ref_cosTheta = (Double_t *)malloc(refEntries_cosTheta*sizeof(Double_t));
	// sort the vectors:
	sort(vec_ref_mHat.begin(), vec_ref_mHat.end());
	sort(vec_ref_cosTheta.begin(), vec_ref_cosTheta.end());
	// fill the sorted arrays:
        for (int j=0 ; j<(int)vec_ref_mHat.size() ; ++j) { array_ref_mHat[j] = vec_ref_mHat[j]; }
	for (int j=0 ; j<(int)vec_ref_cosTheta.size() ; ++j) { array_ref_cosTheta[j] = vec_ref_cosTheta[j]; }

	
	vector<Double_t> vec_SM_mHat; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	vector<Double_t> vec_SM_cosTheta; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	for (int k=0 ; k<Nleptons_input_SM ; k++) {
		invariantMass = anlz.invMass(SM_allInputGoodLeptons[k], SM_allInputGoodaLeptons[k]);
		cosThetaBoost = anlz.cosThetaBoost(SM_allInputGoodLeptons[k], SM_allInputGoodaLeptons[k]);
		cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(SM_allInputGoodLeptons[k], SM_allInputGoodaLeptons[k]);
		if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
		else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
		// fill the vectors
		vec_SM_mHat.push_back(invariantMass);
		if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
			vec_SM_cosTheta.push_back(cosTh);
		}
	}
	// clear the pairs vectors
	anlz.clearAllLeptons();
	Int_t SM_Entries_mHat = (Int_t)vec_SM_mHat.size();
	Int_t SM_Entries_cosTheta = (Int_t)vec_SM_cosTheta.size();
	Double_t *array_SM_mHat = (Double_t *)malloc(SM_Entries_mHat*sizeof(Double_t));
	Double_t *array_SM_cosTheta = (Double_t *)malloc(SM_Entries_cosTheta*sizeof(Double_t));
	// sort the vectors:
	sort(vec_SM_mHat.begin(), vec_SM_mHat.end());
	sort(vec_SM_cosTheta.begin(), vec_SM_cosTheta.end());
	// fill the sorted arrays:
	for (int j=0 ; j<(int)vec_SM_mHat.size() ; ++j) { array_SM_mHat[j] = vec_SM_mHat[j]; }
	for (int j=0 ; j<(int)vec_SM_cosTheta.size() ; ++j) { array_SM_cosTheta[j] = vec_SM_cosTheta[j]; }


	vector<Double_t> vec_E6_mHat; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	vector<Double_t> vec_E6_cosTheta; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	for (int k=0 ; k<Nleptons_input_E6 ; k++) {
		invariantMass = anlz.invMass(E6_allInputGoodLeptons[k], E6_allInputGoodaLeptons[k]);
		cosThetaBoost = anlz.cosThetaBoost(E6_allInputGoodLeptons[k], E6_allInputGoodaLeptons[k]);
		cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(E6_allInputGoodLeptons[k], E6_allInputGoodaLeptons[k]);
		if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
		else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
		// fill the vectors
		vec_E6_mHat.push_back(invariantMass);
		if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
			vec_E6_cosTheta.push_back(cosTh);
		}
	}
	// clear the pairs vectors
	anlz.clearAllLeptons();
	Int_t E6_Entries_mHat = (Int_t)vec_E6_mHat.size();
	Int_t E6_Entries_cosTheta = (Int_t)vec_E6_cosTheta.size();
	Double_t *array_E6_mHat = (Double_t *)malloc(E6_Entries_mHat*sizeof(Double_t));
	Double_t *array_E6_cosTheta = (Double_t *)malloc(E6_Entries_cosTheta*sizeof(Double_t));
	// sort the vectors
	sort(vec_E6_mHat.begin(), vec_E6_mHat.end());
        sort(vec_E6_cosTheta.begin(), vec_E6_cosTheta.end());
	// fill the sorted arrays:
        for (int j=0 ; j<(int)vec_E6_mHat.size() ; ++j) { array_E6_mHat[j] = vec_E6_mHat[j]; }
        for (int j=0 ; j<(int)vec_E6_cosTheta.size() ; ++j) { array_E6_cosTheta[j] = vec_E6_cosTheta[j]; }

	
	vector<Double_t> vec_KK_mHat; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	vector<Double_t> vec_KK_cosTheta; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	for (int k=0 ; k<Nleptons_input_KK ; k++) {
		invariantMass = anlz.invMass(KK_allInputGoodLeptons[k], KK_allInputGoodaLeptons[k]);
		cosThetaBoost = anlz.cosThetaBoost(KK_allInputGoodLeptons[k], KK_allInputGoodaLeptons[k]);
		cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(KK_allInputGoodLeptons[k], KK_allInputGoodaLeptons[k]);
		if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
		else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
		// fill the vectors
		vec_KK_mHat.push_back(invariantMass);
		if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
			vec_KK_cosTheta.push_back(cosTh);
		}
	}
	// clear the pairs vectors
	anlz.clearAllLeptons();
	Int_t KK_Entries_mHat = (Int_t)vec_KK_mHat.size();
	Int_t KK_Entries_cosTheta = (Int_t)vec_KK_cosTheta.size();
	Double_t *array_KK_mHat = (Double_t *)malloc(KK_Entries_mHat*sizeof(Double_t));
	Double_t *array_KK_cosTheta = (Double_t *)malloc(KK_Entries_cosTheta*sizeof(Double_t));
	// sort the vectors
	sort(vec_KK_mHat.begin(), vec_KK_mHat.end());
        sort(vec_KK_cosTheta.begin(), vec_KK_cosTheta.end());
	// fill the sorted arrays:
        for (int j=0 ; j<(int)vec_KK_mHat.size() ; ++j) { array_KK_mHat[j] = vec_KK_mHat[j]; }
        for (int j=0 ; j<(int)vec_KK_cosTheta.size() ; ++j) { array_KK_cosTheta[j] = vec_KK_cosTheta[j]; }

	
	// do the unbinned ks test:
        Double_t unbinnedKStest_SM_mHat = TMath::KolmogorovTest((Int_t)vec_ref_mHat.size(), array_ref_mHat, (Int_t)vec_SM_mHat.size(), array_SM_mHat ,"");
	Double_t unbinnedKStest_E6_mHat = TMath::KolmogorovTest((Int_t)vec_ref_mHat.size(), array_ref_mHat, (Int_t)vec_E6_mHat.size(), array_E6_mHat ,"");
	Double_t unbinnedKStest_KK_mHat = TMath::KolmogorovTest((Int_t)vec_ref_mHat.size(), array_ref_mHat, (Int_t)vec_KK_mHat.size(), array_KK_mHat ,"");

	Double_t unbinnedKStest_SM_cosTheta = TMath::KolmogorovTest((Int_t)vec_ref_cosTheta.size(), array_ref_cosTheta, (Int_t)vec_SM_cosTheta.size(), array_SM_cosTheta ,"");
	Double_t unbinnedKStest_E6_cosTheta = TMath::KolmogorovTest((Int_t)vec_ref_cosTheta.size(), array_ref_cosTheta, (Int_t)vec_E6_cosTheta.size(), array_E6_cosTheta ,"");
	Double_t unbinnedKStest_KK_cosTheta = TMath::KolmogorovTest((Int_t)vec_ref_cosTheta.size(), array_ref_cosTheta, (Int_t)vec_KK_cosTheta.size(), array_KK_cosTheta ,"");

	cout << endl;
	cout << "--------------------------------------------------------"<< endl;
        cout << "mHat     :   SM unbinned KStest = " << unbinnedKStest_SM_mHat << endl;
	cout << "mHat     :   E6 unbinned KStest = " << unbinnedKStest_E6_mHat << endl;
	cout << "mHat     :   KK unbinned KStest = " << unbinnedKStest_KK_mHat << endl;

        cout << "cosTheta :   SM unbinned KStest = " << unbinnedKStest_SM_cosTheta << endl;
	cout << "cosTheta :   E6 unbinned KStest = " << unbinnedKStest_E6_cosTheta << endl;
	cout << "cosTheta :   KK unbinned KStest = " << unbinnedKStest_KK_cosTheta << endl;
        cout << "--------------------------------------------------------" << endl;
	cout << endl;





        // declaration of histograms
        //---------------------------------------------------------------------------------------------------------

        // titles for the cos(theta) plots
        string title  = "";
        string xtitle = "Number of bins";
        string ytitle = "#left(#frac{dN}{dcos#theta '}#right)";

        TH1 *hRef   = NULL;
        TH1 *KK_hInput = NULL;
	TH1 *E6_hInput = NULL;
	TH1 *SM_hInput = NULL;
	
        //----------------------------------------------------------------------------------------------------------

	
	




Double_t maxBins = 32.;
	Double_t minBins = 1.;
	Int_t nGraphEntries = (Int_t)maxBins - (Int_t)minBins;

	cout << "nGraphEntries=" << nGraphEntries << endl; 
	Double_t *nBins               = (Double_t *)malloc(nGraphEntries*sizeof(Double_t));
        Double_t *KK_KolmogorovResult = (Double_t *)malloc(nGraphEntries*sizeof(Double_t));
	Double_t *E6_KolmogorovResult = (Double_t *)malloc(nGraphEntries*sizeof(Double_t));
	Double_t *SM_KolmogorovResult = (Double_t *)malloc(nGraphEntries*sizeof(Double_t));



	Double_t KK_Kolmogorov = 0.;
	Double_t E6_Kolmogorov = 0.;
	Double_t SM_Kolmogorov = 0.;	
	if (!doRun) {
		delete hRef;
		delete KK_hInput;
		delete E6_hInput;
		delete SM_hInput;

		// binnings - corresponding to the best Kolmogorov test
		// 100[fb^-1] cosTheta
		//    -- SM ref 22 bins v
		//    -- E6 ref 16 bins v
		//    -- KK ref 22 bins v
		// 100[fb^-1] mHat
		//    -- SM ref 20 bins v
		//    -- E6 ref 12 bins v
		//    -- KK ref 20 bins v
		// 500[fb^-1] cosTheta
		//    -- SM ref 20 bins v
		//    -- E6 ref 20 bins v
		//    -- KK ref 20 bins v
		// 500[fb^-1] mHat
		//    -- SM ref 14 bins v
		//    -- E6 ref 20 bins v
		//    -- KK ref 20 bins v

		// boundries and bins of histograms
		Int_t invMnBins = 50;
		Int_t cosThetanBins = 22;

		// choose mHat & cosTheta binnings
		switch(inputSampleSizeSelector) {
			case 1: invMnBins = 50; cosThetanBins = 22;
				if (errorSelector==1) {
					invMnBins -= 1;
					cosThetanBins -= 1;
				}
				else if (errorSelector==2) {
					invMnBins += 1;
					cosThetanBins += 1;
				}
				break; // 1M sample
			case 2: // 500[fb^-1] sample
				switch(iRefModel) {
					case 1: invMnBins = 20; cosThetanBins = 20; break; // KK ref
					case 2: invMnBins = 14; cosThetanBins = 20; break; // SM ref
					case 3: invMnBins = 20; cosThetanBins = 20; break; // E6 ref
					default: break;
				}
				if (errorSelector==1) {
					invMnBins -= 1;
					cosThetanBins -= 1;	
				}
				else if (errorSelector==2) {
                                        invMnBins += 1;
                                        cosThetanBins += 1;
                                }
				break;
			case 3: // 100[fb^-1] sample
				switch(iRefModel) {
					case 1: invMnBins = 20; cosThetanBins = 22; break; // KK ref
					case 2: invMnBins = 20; cosThetanBins = 22; break; // SM ref
					case 3: invMnBins = 12; cosThetanBins = 16; break; // E6 ref
					default: break;
				}
				if (errorSelector==1) {
					invMnBins -= 1;
					cosThetanBins -= 1;
				}
				else if (errorSelector==2) {
					invMnBins += 1;
					cosThetanBins += 1;
				}
				break;
			default: break;
		}
		
		switch(distributionSelector) {
			case 1:
				hRef      = new TH1D("", "", invMnBins, min, max);
				KK_hInput = new TH1D("", "", invMnBins, min, max);
				E6_hInput = new TH1D("", "", invMnBins, min, max);
				SM_hInput = new TH1D("", "", invMnBins, min, max);
				break;
			case 2:
				hRef      = new TH1D("", "", cosThetanBins, min, max);
				KK_hInput = new TH1D("", "", cosThetanBins, min, max);
				E6_hInput = new TH1D("", "", cosThetanBins, min, max);
				SM_hInput = new TH1D("", "", cosThetanBins, min, max);
				break;
			case 3:
				hRef      = new TH1D("", "", 20, min, max);
				KK_hInput = new TH1D("", "", 20, min, max);
				E6_hInput = new TH1D("", "", 20, min, max);
				SM_hInput = new TH1D("", "", 20, min, max);
				break;
			default: exit(-1);
		}
		
		// run over the good pairs (passed whatever cuts applied before) and fill the histograms
			for (int k=0 ; k<Nleptons_ref ; k++) {
				invariantMass = anlz.invMass(allRefGoodLeptons[k], allRefGoodaLeptons[k]);
				pT = anlz.pT(allRefGoodLeptons[k], allRefGoodaLeptons[k]);
				cosThetaBoost = anlz.cosThetaBoost(allRefGoodLeptons[k], allRefGoodaLeptons[k]);
				if (quarkDirectionOrRapidity==2) {
					cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allRefGoodLeptons[k], allRefGoodaLeptons[k]);
				}
				else if (quarkDirectionOrRapidity==1) {
					cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allRefGoodLeptons[k], allRefGoodaLeptons[k], true);
				}
				else {cout << "wrong option for quarkDirectionOrRapidity: " << quarkDirectionOrRapidity << " exitting now" << endl; exit(1);}
				if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
				else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
				// fill the histos
				switch(distributionSelector) {
					case 1:
						hRef->Fill( invariantMass );
						break;
					case 2:
						if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
							hRef->Fill( cosTh );
						}
						break;
					case 3:
						hRef->Fill( pT );
						break;
					default: exit(-1);
				}
			}
			//      clear the pairs vectors
			anlz.clearAllLeptons();		

			//---------------------------------------------------------------------------------------------------
			// run over the good pairs (passed whatever cuts applied before) and fill the histograms
			for (int k=0 ; k<Nleptons_input_KK ; k++) {
				invariantMass = anlz.invMass(KK_allInputGoodLeptons[k], KK_allInputGoodaLeptons[k]);
				pT = anlz.pT(KK_allInputGoodLeptons[k], KK_allInputGoodaLeptons[k]);
				cosThetaBoost = anlz.cosThetaBoost(KK_allInputGoodLeptons[k], KK_allInputGoodaLeptons[k]);
				if (quarkDirectionOrRapidity==2) {
					cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(KK_allInputGoodLeptons[k], KK_allInputGoodaLeptons[k]);
				}
				else if (quarkDirectionOrRapidity==1) {
					cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(KK_allInputGoodLeptons[k], KK_allInputGoodaLeptons[k], true);
				}
				else {cout << "wrong option for quarkDirectionOrRapidity: " << quarkDirectionOrRapidity << " exitting now" << endl; exit(1);}
				if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
				else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
				// fill the histos
				switch(distributionSelector) {
                                        case 1:
                                                KK_hInput->Fill( invariantMass );
                                                break;
                                        case 2:
                                                if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
                                                        KK_hInput->Fill( cosTh );
                                                }
                                                break;
                                        case 3:
                                                KK_hInput->Fill( pT );
                                                break;
                                        default: exit(-1);
                                }
			}
			// clear the pairs vectors
			anlz.clearAllLeptons();
			//---------------------------------------------------------------------------------------------------
			// run over the good pairs (passed whatever cuts applied before) and fill the histograms
                        for (int k=0 ; k<Nleptons_input_E6 ; k++) {
                                invariantMass = anlz.invMass(E6_allInputGoodLeptons[k], E6_allInputGoodaLeptons[k]);
				pT = anlz.pT(E6_allInputGoodLeptons[k], E6_allInputGoodaLeptons[k]);
                                cosThetaBoost = anlz.cosThetaBoost(E6_allInputGoodLeptons[k], E6_allInputGoodaLeptons[k]);
                                if (quarkDirectionOrRapidity==2) {
                                        cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(E6_allInputGoodLeptons[k], E6_allInputGoodaLeptons[k]);
                                }
                                else if (quarkDirectionOrRapidity==1) {
                                        cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(E6_allInputGoodLeptons[k], E6_allInputGoodaLeptons[k], true);
                                }
                                else {cout << "wrong option for quarkDirectionOrRapidity: " << quarkDirectionOrRapidity << " exitting now" << endl; exit(1);}
                                if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
                                else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
                                // fill the histos
				switch(distributionSelector) {
                                        case 1:
                                                E6_hInput->Fill( invariantMass );
                                                break;
                                        case 2:
                                                if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
                                                        E6_hInput->Fill( cosTh );
                                                }
                                                break;
                                        case 3:
                                                E6_hInput->Fill( pT );
                                                break;
                                        default: exit(-1);
                                }
                        }
                        // clear the pairs vectors
                        anlz.clearAllLeptons();
			// run over the good pairs (passed whatever cuts applied before) and fill the histograms
                        for (int k=0 ; k<Nleptons_input_SM ; k++) {
                                invariantMass = anlz.invMass(SM_allInputGoodLeptons[k], SM_allInputGoodaLeptons[k]);
				pT = anlz.pT(SM_allInputGoodLeptons[k], SM_allInputGoodaLeptons[k]);
                                cosThetaBoost = anlz.cosThetaBoost(SM_allInputGoodLeptons[k], SM_allInputGoodaLeptons[k]);
                                if (quarkDirectionOrRapidity==2) {
                                        cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(SM_allInputGoodLeptons[k], SM_allInputGoodaLeptons[k]);
                                }
                                else if (quarkDirectionOrRapidity==1) {
                                        cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(SM_allInputGoodLeptons[k], SM_allInputGoodaLeptons[k], true);
                                }
                                else {cout << "wrong option for quarkDirectionOrRapidity: " << quarkDirectionOrRapidity << " exitting now" << endl; exit(1);}
                                if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
                                else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
                                // fill the histos
				switch(distributionSelector) {
                                        case 1:
                                                SM_hInput->Fill( invariantMass );
                                                break;
                                        case 2:
                                                if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
                                                        SM_hInput->Fill( cosTh );
                                                }
                                                break;
                                        case 3:
                                                SM_hInput->Fill( pT );
                                                break;
                                        default: exit(-1);
                                }
                        }
                        // clear the pairs vectors
                        anlz.clearAllLeptons();
			//---------------------------------------------------------------------------------------------------

			// make the tests and put them into graph
			KK_Kolmogorov = KK_hInput->KolmogorovTest(hRef,"D");
			E6_Kolmogorov = E6_hInput->KolmogorovTest(hRef,"D");
			SM_Kolmogorov = SM_hInput->KolmogorovTest(hRef,"D");
			cout << "KK_Kolmogorov = " << KK_Kolmogorov << endl;
			cout << "E6_Kolmogorov = " << E6_Kolmogorov << endl;
			cout << "SM_Kolmogorov = " << SM_Kolmogorov << endl;
	}

	else if (doRun) {
		cout << "start running..." << "\n" << endl;
		for (Int_t n=0 ; n<nGraphEntries ; n++) {

			delete hRef;
			delete KK_hInput;
			delete E6_hInput;
			delete SM_hInput;
	
			nBins[n] = n + minBins;

			hRef      = new TH1D("", "", n, min, max);
			KK_hInput = new TH1D("", "", n, min, max);
			E6_hInput = new TH1D("", "", n, min, max);
			SM_hInput = new TH1D("", "", n, min, max);
		
			// run over the good pairs (passed whatever cuts applied before) and fill the histograms
			for (int k=0 ; k<Nleptons_ref ; k++) {
				invariantMass = anlz.invMass(allRefGoodLeptons[k], allRefGoodaLeptons[k]);
				pT = anlz.pT(allRefGoodLeptons[k], allRefGoodaLeptons[k]);
				cosThetaBoost = anlz.cosThetaBoost(allRefGoodLeptons[k], allRefGoodaLeptons[k]);
				if (quarkDirectionOrRapidity==2) {
					cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allRefGoodLeptons[k], allRefGoodaLeptons[k]);
				}
				else if (quarkDirectionOrRapidity==1) {
					cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allRefGoodLeptons[k], allRefGoodaLeptons[k], true);
				}
				else {cout << "wrong option for quarkDirectionOrRapidity: " << quarkDirectionOrRapidity << " exitting now" << endl; exit(1);}
				if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
				else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
				// fill the histos
				switch(distributionSelector) {
					case 1:
						hRef->Fill( invariantMass );
						break;
					case 2:
						if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
							hRef->Fill( cosTh );
						}
						break;
					case 3:
						hRef->Fill( pT );
						break;
					default: exit(-1);
				}
			}
			//      clear the pairs vectors
			anlz.clearAllLeptons();		

			//---------------------------------------------------------------------------------------------------
			// run over the good pairs (passed whatever cuts applied before) and fill the histograms
			for (int k=0 ; k<Nleptons_input_KK ; k++) {
				invariantMass = anlz.invMass(KK_allInputGoodLeptons[k], KK_allInputGoodaLeptons[k]);
				pT = anlz.pT(KK_allInputGoodLeptons[k], KK_allInputGoodaLeptons[k]);
				cosThetaBoost = anlz.cosThetaBoost(KK_allInputGoodLeptons[k], KK_allInputGoodaLeptons[k]);
				if (quarkDirectionOrRapidity==2) {
					cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(KK_allInputGoodLeptons[k], KK_allInputGoodaLeptons[k]);
				}
				else if (quarkDirectionOrRapidity==1) {
					cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(KK_allInputGoodLeptons[k], KK_allInputGoodaLeptons[k], true);
				}
				else {cout << "wrong option for quarkDirectionOrRapidity: " << quarkDirectionOrRapidity << " exitting now" << endl; exit(1);}
				if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
				else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
				// fill the histos
				switch(distributionSelector) {
                                        case 1:
                                                KK_hInput->Fill( invariantMass );
                                                break;
                                        case 2:
                                                if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
                                                        KK_hInput->Fill( cosTh );
                                                }
                                                break;
                                        case 3:
                                                KK_hInput->Fill( pT );
                                                break;
                                        default: exit(-1);
                                }
			}
			// clear the pairs vectors
			anlz.clearAllLeptons();
			//---------------------------------------------------------------------------------------------------
			// run over the good pairs (passed whatever cuts applied before) and fill the histograms
                        for (int k=0 ; k<Nleptons_input_E6 ; k++) {
                                invariantMass = anlz.invMass(E6_allInputGoodLeptons[k], E6_allInputGoodaLeptons[k]);
				pT = anlz.pT(E6_allInputGoodLeptons[k], E6_allInputGoodaLeptons[k]);
                                cosThetaBoost = anlz.cosThetaBoost(E6_allInputGoodLeptons[k], E6_allInputGoodaLeptons[k]);
                                if (quarkDirectionOrRapidity==2) {
                                        cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(E6_allInputGoodLeptons[k], E6_allInputGoodaLeptons[k]);
                                }
                                else if (quarkDirectionOrRapidity==1) {
                                        cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(E6_allInputGoodLeptons[k], E6_allInputGoodaLeptons[k], true);
                                }
                                else {cout << "wrong option for quarkDirectionOrRapidity: " << quarkDirectionOrRapidity << " exitting now" << endl; exit(1);}
                                if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
                                else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
                                // fill the histos
				switch(distributionSelector) {
                                        case 1:
                                                E6_hInput->Fill( invariantMass );
                                                break;
                                        case 2:
                                                if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
                                                        E6_hInput->Fill( cosTh );
                                                }
                                                break;
                                        case 3:
                                                E6_hInput->Fill( pT );
                                                break;
                                        default: exit(-1);
                                }
                        }
                        // clear the pairs vectors
                        anlz.clearAllLeptons();
			// run over the good pairs (passed whatever cuts applied before) and fill the histograms
                        for (int k=0 ; k<Nleptons_input_SM ; k++) {
                                invariantMass = anlz.invMass(SM_allInputGoodLeptons[k], SM_allInputGoodaLeptons[k]);
				pT = anlz.pT(SM_allInputGoodLeptons[k], SM_allInputGoodaLeptons[k]);
                                cosThetaBoost = anlz.cosThetaBoost(SM_allInputGoodLeptons[k], SM_allInputGoodaLeptons[k]);
                                if (quarkDirectionOrRapidity==2) {
                                        cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(SM_allInputGoodLeptons[k], SM_allInputGoodaLeptons[k]);
                                }
                                else if (quarkDirectionOrRapidity==1) {
                                        cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(SM_allInputGoodLeptons[k], SM_allInputGoodaLeptons[k], true);
                                }
                                else {cout << "wrong option for quarkDirectionOrRapidity: " << quarkDirectionOrRapidity << " exitting now" << endl; exit(1);}
                                if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
                                else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
                                // fill the histos
				switch(distributionSelector) {
                                        case 1:
                                                SM_hInput->Fill( invariantMass );
                                                break;
                                        case 2:
                                                if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
                                                        SM_hInput->Fill( cosTh );
                                                }
                                                break;
                                        case 3:
                                                SM_hInput->Fill( pT );
                                                break;
                                        default: exit(-1);
                                }
                        }
                        // clear the pairs vectors
                        anlz.clearAllLeptons();
			//---------------------------------------------------------------------------------------------------

			// make the tests and put them into graph
			KK_KolmogorovResult[n] = KK_hInput->KolmogorovTest(hRef/*,"D"*/);
			E6_KolmogorovResult[n] = E6_hInput->KolmogorovTest(hRef/*,"D"*/);
			SM_KolmogorovResult[n] = SM_hInput->KolmogorovTest(hRef/*,"D"*/);
	

			SM_kolmogorovTest = SM_KolmogorovResult[n];
			E6_kolmogorovTest = E6_KolmogorovResult[n];
			KK_kolmogorovTest = KK_KolmogorovResult[n];
			numberOfBins      = (Int_t)nBins[n];
			outTree->Fill();

			cout  << "n=" << n << "/" << nGraphEntries << "\r" << flush;
		}
		outTree->Write();
 		outTreeFile->Write();
	}
	cout << "\n" << endl;
	cout << "...end of run" << "\n" << endl;
	if (allRefGoodLeptons.size() != 0) { allRefGoodLeptons.clear(); }
	if (allRefGoodaLeptons.size() != 0) { allRefGoodaLeptons.clear(); }
	if (KK_allInputGoodLeptons.size() != 0) { KK_allInputGoodLeptons.clear(); }
	if (KK_allInputGoodaLeptons.size() != 0) { KK_allInputGoodaLeptons.clear(); }
	if (E6_allInputGoodLeptons.size() != 0) { E6_allInputGoodLeptons.clear(); }
        if (E6_allInputGoodaLeptons.size() != 0) { E6_allInputGoodaLeptons.clear(); }
	if (SM_allInputGoodLeptons.size() != 0) { SM_allInputGoodLeptons.clear(); }
        if (SM_allInputGoodaLeptons.size() != 0) { SM_allInputGoodaLeptons.clear(); }

	// make the graph
	TGraph *KK_KolmogorovGraph = new TGraph(nGraphEntries, nBins, KK_KolmogorovResult);
	TGraph *E6_KolmogorovGraph = new TGraph(nGraphEntries, nBins, E6_KolmogorovResult);
	TGraph *SM_KolmogorovGraph = new TGraph(nGraphEntries, nBins, SM_KolmogorovResult);


	KK_KolmogorovGraph->SetLineStyle(1);
	KK_KolmogorovGraph->SetTitle("");
	KK_KolmogorovGraph->SetMarkerStyle(20);
//	KolmogorovGraph->GetXaxis()->SetTitle(xtitle.c_str());
//	KolmogorovGraph->GetYaxis()->SetTitle("Kolmogorov Test");

        E6_KolmogorovGraph->SetLineStyle(3);
        E6_KolmogorovGraph->SetTitle("");
        E6_KolmogorovGraph->SetMarkerStyle(30);
//      KolmogorovGraph->GetXaxis()->SetTitle(xtitle.c_str());
//      KolmogorovGraph->GetYaxis()->SetTitle("Kolmogorov Test");
	
        SM_KolmogorovGraph->SetLineStyle(5);
        SM_KolmogorovGraph->SetTitle("");
        SM_KolmogorovGraph->SetMarkerStyle(40);
//      KolmogorovGraph->GetXaxis()->SetTitle(xtitle.c_str());
//      KolmogorovGraph->GetYaxis()->SetTitle("Kolmogorov Test");




	// Kolmogorov test:
	cout << "Kolmogorov test result: probability of compatibility between the 2 histograms." << endl;
	cout << "The returned function value is the probability of test (much less than one means NOT compatible)" << "\n" << endl;


	// ------ legends -----------------------------------------------------------------------------------------------------
	TLegend *leg = new TLegend(0.45,0.25,0.75,0.45);
	if (iRefModel == 3) {sRefModel = "Z'_{SM}"; }
	string inputLuminosity = "";
	string refLuminosity   = "";
	if (inputSampleSizeSelector==1) {
		inputLuminosity = "L>>[fb^{-1}]";
	}
	else if (inputSampleSizeSelector==2) {
		inputLuminosity = "L=500[fb^{-1}]";
	}
	else if (inputSampleSizeSelector==3) {
		inputLuminosity = "L=100[fb^{-1}]";
	}
	if (refSampleSizeSelector==1) {
		refLuminosity = "L>>[fb^{-1}]";
	}
	else if (refSampleSizeSelector==2) {
		refLuminosity = "L=500[fb^{-1}]";
	}
	else if (refSampleSizeSelector==3) {
		refLuminosity = "L=100[fb^{-1}]";
	}

	string label1 = "SM#left(" + inputLuminosity + "#right) vs. " + sRefModel + "#left(" + refLuminosity + "#right)";
	string label2 = "KK#left(" + inputLuminosity + "#right) vs. " + sRefModel + "#left(" + refLuminosity + "#right)";
	string label3 = "Z'_{SM}#left(" + inputLuminosity + "#right) vs. " + sRefModel + "#left(" + refLuminosity + "#right)";
	//if (iRefModel ==3) {sRefModel = "Z'"; }
	//if (iInputModel ==3) {sInputModel = "Z'"; }
	//string label1 = "Kolmogorov Test #left(" + sInputModel + " vs. " + sRefModel + "#right)";
	//string label2 = "#chi^{2} Test #left(" + sInputModel + " vs. " + sRefModel + "#right)";
	leg->AddEntry(SM_KolmogorovGraph,label1.c_str(),"L"/*"P"*/);
	leg->AddEntry(KK_KolmogorovGraph,label2.c_str(),"L"/*"P"*/);
	leg->AddEntry(E6_KolmogorovGraph,label3.c_str(),"L"/*"P"*/);
	//leg->AddEntry(Chi2Graph,label2.c_str(),"L");

	//TPaveText *pav = new TPaveText(0.15,0.45,0.35,0.65);
	//pav->AddText(text1.c_str());
	//pav->AddText(text2.c_str());



	// frames
	TH2 *frameHisto = new TH2D("", "", 1,minBins,maxBins, 1,0.,1.15);
	frameHisto->SetXTitle("Number of bins");
//	frameHisto->SetYTitle("Kolmogorov & #chi^{2} Tests");
	frameHisto->SetYTitle(yTitle.c_str());

	cout << "\n---finalizing---\n" << endl;

	// draw the histos
	if (doRun) {
		frameHisto->Draw();
		KK_KolmogorovGraph->Draw("L");
		E6_KolmogorovGraph->Draw("L");
		SM_KolmogorovGraph->Draw("L");
		leg->Draw("sames");
		//pav->Draw("sames");

	
		// finalize the graphics
		canv->Update();
		canv->Print(Kolmogorov_RootFileName.c_str(),"root");
		canv->Update();
		canv->Print(Kolmogorov_EpsFileName.c_str());
		Kolmogorov_HistFile->Write();

		// wait so the plots can hold on
		canv->WaitPrimitive();
	}

	// Done.
	return 0;
}

