/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;
using namespace Moses1;
using namespace Pythia8;
//using namespace RooFit;
#define P1 28 //SM=30 //KK=28 //E6=20
#define P2 30 //SM=32 //KK=30 //E6=22
#define P3 32 //SM=34 //KK=32 //E6=24


Double_t powN100(Double_t x, Int_t N) {
	Double_t tmp = x;
	if (abs(N)>100) {cerr << "|N|>100 is not supported" << endl; exit(-1);}
	switch(abs(N)) {
		case 0:
			if (tmp == 0.) {cerr << "0^0 is undefined" << endl; exit(-1);}
			else return 1.;
		case 1: return tmp;
		default:
			for (int i=2 ; i<=N ; i++) { tmp *= x; }
			break;
	}
	if (N<0) {return 1./tmp;}
	else {return tmp;}
}


Double_t fitFunction(Double_t *x, Double_t *par) {
	if (x[0] <= 1. && x[0] >= -1.) {
//		return 1. - par[0] * pow((x[0]-par[2]), par[1]) - par[3] * pow((x[0]-par[5]), par[4]) - par[6] * pow((x[0]-par[8]), par[7]);
//		return 1. - par[0] * powN100(x[0]-par[1], P1) - par[2] * powN100(x[0]-par[3], P2) - par[4] * powN100(x[0]-par[5], P3);
		return 1. - par[0] * powN100(x[0], P1) - par[1] * powN100(x[0], P2) - par[2] * powN100(x[0], P3);
	}
	else {return 0.;}
}


int main(int argc, char* argv[]) {
	cout << "Holy Moses..." << endl;
	cout << "!!!!!!!! Remember to change (and recompile) the global parameters P1, P2 and P3 according to the selected model !!!!!!!!" << endl;

	// set styles
	TStyle* s = (TStyle*)gROOT->GetStyle("Plain");
	s->cd();
	gStyle->SetOptStat(11);

	// declare the application
	TApplication theApp("App", &argc, argv);

	commonOperations commonOps; // set the class for commonOperations call
	// set styles
	commonOps.SetOKStyle();

	// set the prefix
//	string dirName = "/KKvsZE6vsSMstudy/tmp/";
	string dirName = "/KKvsZE6vsSMstudy/";
	int iModel = 1;
	string sModel = "KK";
	cout << "KK(1), SM(2) or E6(3) ?" << endl;
	cout << "..." << endl;
	cin >> iModel;
	switch(iModel) {
		case 1: sModel = "KK"; break;
		case 2: sModel = "SM"; break;
		case 3: sModel = "E6"; break;
		default: cout << "wrong option, set to default: KK(1)" << endl;
			sModel = "KK"; break;
			break;
	}
	string outFileName = dirName + sModel + "_selectionEfficiency";


	int sampleSizeSelector = 1;
	cout << "select sample size: 1M(1), L=100fb^-1(2), L=500fb^-1(3)..." << endl;
	cin >> sampleSizeSelector;
	if (sampleSizeSelector!=1 && sampleSizeSelector!=2 && sampleSizeSelector!=3) {
		cout << "invalid option, set to 1 (default)" << endl;
		sampleSizeSelector = 1;
	}




	//--- variables to put in the trees for RooFit
	Int_t    eventNumber = 0;
	Double_t costheta_forRooFitTree;

	// declare the .root file for the trees for RooFit
	string cosThetaRootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName+"_forRooFit.root");
	TFile *roo_treeFile = new TFile(cosThetaRootFileName.c_str(),"recreate");
	TTree *tree_SM = new TTree("tree_SM","tree_SM");
	TTree *tree_E6 = new TTree("tree_E6","tree_E6");
	TTree *tree_KK = new TTree("tree_KK","tree_KK");

	//--- trees to put cos(theta) in
	if (sModel=="SM") {
		tree_SM->Branch("roo_eventNumber",&eventNumber,"eventNumber/I");
		tree_SM->Branch("roo_cosTheta",&costheta_forRooFitTree,"cosTheta_forRooFitTree/D");
	}
	else if (sModel=="E6") {
		tree_E6->Branch("roo_eventNumber",&eventNumber,"eventNumber/I");
		tree_E6->Branch("roo_cosTheta",&costheta_forRooFitTree,"cosTheta_forRooFitTree/D");
	}
	else if (sModel=="KK") {
		tree_KK->Branch("roo_eventNumber",&eventNumber,"eventNumber/I");
		tree_KK->Branch("roo_cosTheta",&costheta_forRooFitTree,"cosTheta_forRooFitTree/D");
	}




	// declare the relevant classes
	cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
	cutsEmulator cuts; // create the cuts emulator instance (input to rapidity)
	dileptonAnalyzer anlz;
	Particle particle;

	// set cuts
	double invMassCut=1000.;
	double pTransverseCut=10.;
	double EtaCut=2.5;

	double no_invMassCut=0.;
	double no_pTransverseCut=0.;
	double no_EtaCut=1e10;


	// choose reference frame
	int referenceFrameSelector = 0;
	cout << "choose reference frame to calculate cos(theta)" << endl;
	cout << "(1) Helicity frame or (2) Collins-Soper frame" << endl;
	cout << "..." << endl;
	cin >> referenceFrameSelector;
	if (referenceFrameSelector != 1 && referenceFrameSelector != 2) {
		cout << "invalid option, setting to default = 1" << endl;
		referenceFrameSelector = 1;
	}
	string cosThetaTitle;
	if (referenceFrameSelector == 1) {cosThetaTitle = "Helicity";}
	else if (referenceFrameSelector == 2) {cosThetaTitle = "Collins-Soper";}

	



	// declare kinematics histograms and root items
	string selectionEfficiencyRootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName+".root");
	string selectionEfficiencyEpsFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName+".eps");


	TFile *selectionEfficiencyHistFile = new TFile(selectionEfficiencyRootFileName.c_str(),"recreate");
	TCanvas *canv = new TCanvas("canv","Kaluza-Klein selection efficiency",0,0,1400,800);
	canv->Divide(1,2);

	// plots of analysis on muon pairs
	TVirtualPad *pad1=canv->cd(1);
//	pad1->SetLogy();
	
	TVirtualPad *pad2=canv->cd(2);
//	pad2->SetLogy();


	Int_t    cosThetanBins = 200; // 60
	Double_t cosThetamin   = -1.;
	Double_t cosThetamax   = 1.;



	
	// titles for the cos(theta) plots
//	string title  = "SM vs KK vs Z': cosTheta (in " + cosThetaTitle + " frame) *Normalized*";
	string title  = "";
	string xtitle = "cos#theta ' (" + cosThetaTitle + " frame)";
	string ytitle = "#left(#frac{dN}{dcos#theta '}#right)_{cuts} #divide #left(#frac{dN}{dcos#theta '}#right)_{full}";


	// declaration of histograms
	//---------------------------------------------------------------------------------------------------------
	TH1 *cosTheta_withCuts = new TH1D("", "", cosThetanBins, cosThetamin, cosThetamax);
	cosTheta_withCuts->SetLineStyle(1);
	cosTheta_withCuts->SetXTitle(xtitle.c_str());
	cosTheta_withCuts->SetYTitle("Events");
	
	TH1 *cosTheta_noCuts = new TH1D("", "", cosThetanBins, cosThetamin, cosThetamax);
	cosTheta_noCuts->SetLineStyle(3);
	cosTheta_noCuts->SetXTitle(xtitle.c_str());
	cosTheta_noCuts->SetYTitle("Events");

	
	//----------------------------------------------------------------------------------------------------------	
	string filePath;
	int Nevents;
	vector<fullRecord> allgoodLeptons, allgoodaLeptons;
	int Nleptons, Naleptons;
	double cosThetaBoost, cosThetaCollinsSoper;
	double invariantMass, cosTh=0;

	// to check the asymmetry in a narrow range arrownd the peak
	double mZ_known = physPrm.mStar;
	double mZ_lowLimit = physPrm.mStar-2000.;
	double mZ_highLimit = physPrm.mStar+1000.;
	cout << "\n" << "mZ_known=" << mZ_known << "  mZ_lowLimit=" << mZ_lowLimit << "  mZ_highLimit=" << mZ_highLimit << "\n" << endl;
	


	// read the trees and put all good lepton pairs into 2 vectors
//	filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_extractedParticles_2trees"+".root");

	if (sampleSizeSelector==1) { // sample size = 1M
		filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+sModel+"_extractedParticles_2trees_1Msample"+".root");
	}
	else if (sampleSizeSelector==2) { // L=100fb^-1
		if (iModel==1) { // KK
			filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+sModel+"_extractedParticles_2trees_400EventsSample"+".root");
		}
		else if (iModel==2) { // SM
			filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+sModel+"_extractedParticles_2trees_480EventsSample"+".root");
		}
		else if (iModel==3) { //E6
			filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+sModel+"_extractedParticles_2trees_460EventsSample"+".root");
		}
	}
	else if (sampleSizeSelector==3) { // L=500fb^-1
		filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"500inversefemtobarn/"+sModel+"_extractedParticles_2trees_500inversefemtobarnSample.root");
	}
		
	anlz.setCuts(invMassCut, pTransverseCut, EtaCut);
	Nevents = anlz.loopOverEvents(filePath,"testTree");
	cout << "Nevents = " << Nevents << endl;
	// get all the good pairs (after cuts)
	Nleptons = anlz.getAllLeptons(allgoodLeptons);
	Naleptons = anlz.getAllaLeptons(allgoodaLeptons);
	cout << "Nleptons = " << Nleptons << endl;
	if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}
	cout << "\n" << endl;
	
	// run over the good pairs (passed cuts) and fill the histograms
	for (int k=0 ; k<Nleptons ; k++) {
		invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
		cosThetaBoost = anlz.cosThetaBoost(allgoodLeptons[k], allgoodaLeptons[k]);
		cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allgoodLeptons[k], allgoodaLeptons[k]);
		if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
		else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
		// fill the histos
		if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
			cosTheta_withCuts->Fill( (Double_t)cosTh );
			costheta_forRooFitTree = (Double_t)cosTh;
			eventNumber ++;
			if      (sModel=="SM") {tree_SM->Fill();}
			else if (sModel=="E6") {tree_E6->Fill();}
			else if (sModel=="KK") {tree_KK->Fill();}
		}
	}
	// clear the pairs vectors
	anlz.clearAllLeptons();
	if (allgoodLeptons.size() != 0) { allgoodLeptons.clear(); }
	if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }	
		
	//-------------------------------------------------------------------------------------------------------------------
	// no cuts
	anlz.setCuts(no_invMassCut, no_pTransverseCut, no_EtaCut);
	Nevents = anlz.loopOverEvents(filePath,"testTree_allPairs");
	cout << "Nevents_allPairs = " << Nevents << endl;
	// get all the good pairs (no cuts)
	Nleptons = anlz.getAllLeptons(allgoodLeptons);
	Naleptons = anlz.getAllaLeptons(allgoodaLeptons);
	cout << "Nleptons_allPairs = " << Nleptons << endl;
	if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}
	cout << "\n" << endl;

	// run over the good pairs (passed cuts) and fill the histograms
	for (int k=0 ; k<Nleptons ; k++) {
		invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
		cosThetaBoost = anlz.cosThetaBoost(allgoodLeptons[k], allgoodaLeptons[k]);
		cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allgoodLeptons[k], allgoodaLeptons[k]);
		if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
		else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
		// fill the histos
		if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
			cosTheta_noCuts->Fill( (Double_t)cosTh );
		}
	}
	// clear the pairs vectors
	anlz.clearAllLeptons();
	if (allgoodLeptons.size() != 0) { allgoodLeptons.clear(); }
	if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }



	cosTheta_withCuts->Print();
	cosTheta_noCuts->Print();

	roo_treeFile->Write();

	cout << "\n" << endl;




	// arrays
	Double_t *cosThetaAxis = (Double_t *)malloc(cosThetanBins*sizeof(Double_t));
	Double_t *efficiency   = (Double_t *)malloc(cosThetanBins*sizeof(Double_t));
	Double_t *eX           = (Double_t *)malloc(cosThetanBins*sizeof(Double_t));
	Double_t *eY           = (Double_t *)malloc(cosThetanBins*sizeof(Double_t));

	Double_t cosThetaHighBound = 1.;
	Double_t cosThetaLowBound = -1.;
	Double_t withCuts, noCuts, ratio;
	Double_t dCost = (cosThetaHighBound - cosThetaLowBound)/(Double_t)cosThetanBins;
	for (Int_t n=0 ; n<cosThetanBins ; n++) {
		cosThetaAxis[n] = -1. + (1.+2*n)*0.5*dCost;
		eX[n] = 0.5*dCost;
		withCuts = cosTheta_withCuts->GetBinContent(n);
		noCuts   = cosTheta_noCuts->GetBinContent(n);
		if (noCuts!=0) { // for not dividing by zero...
			ratio = withCuts / noCuts;
			efficiency[n] = ratio;
			eY[n] = sqrt(ratio*(1.-ratio)/noCuts);
		}
		else {
			efficiency[n] = 0.;
			eY[n] = 0.;
		}
	}	


        // efficiency graph
	TGraphErrors *efficiencyCostheta = new TGraphErrors(cosThetanBins, cosThetaAxis, efficiency, eX, eY);
	efficiencyCostheta->SetLineStyle(1);
	efficiencyCostheta->SetTitle("");
	efficiencyCostheta->GetXaxis()->SetTitle(xtitle.c_str());
	efficiencyCostheta->GetYaxis()->SetTitle(ytitle.c_str());



	//---------- fit ------------------------------
        stringstream stmTmp;
        string par0Tmp, par1Tmp, par2Tmp, par3Tmp, par4Tmp, par5Tmp, par6Tmp, par7Tmp, par8Tmp;
        string fitFormula;
	Double_t chi2dof = 0.;

//	Int_t power = 0, power1 = 0, power2 = 0;
//	Int_t dpower = 0, dpower1 = 0, dpower2 = 0;
	Double_t norm = 1., norm1 = 1., norm2 = 1.;
        Double_t dnorm = 0., dnorm1 = 0., dnorm2 = 0.;

/*
        Double_t norm = 1., norm1 = 1., norm2 = 1.;
        Double_t dnorm = 0., dnorm1 = 0., dnorm2 = 0.;
	Double_t offset = 0., offset1 = 0., offset2 = 0.;
	Double_t doffset = 0., doffset1 = 0., doffset2 = 0.;
*/

        //create a function in the range [-1,1] with 2 parameters
//	TF1 *func = new TF1("efficiencyFit",fitFunction,-1.,1.,9);
//	TF1 *func = new TF1("efficiencyFit",fitFunction,-1.,1.,6);
	TF1 *func = new TF1("efficiencyFit",fitFunction,-1.,1.,3);
        // initialize parameters

	func->SetParameter(0,1.); // parameter 0
        func->SetParameter(1,1.); // parameter 2
        func->SetParameter(2,1.); // parameter 4

/*
        func->SetParameter(0,1.); // parameter 0
	func->SetParameter(1,20); // parameter 1
	func->SetParameter(2,0.); // parameter 2
	func->SetParameter(3,1.); // parameter 3
	func->SetParameter(4,60); // parameter 4
	func->SetParameter(5,0.); // parameter 5
	func->SetParameter(6,1.); // parameter 6
	func->SetParameter(7,60); // parameter 7
        func->SetParameter(8,0.); // parameter 8
*/
/*
	func->SetParameter(0,1.); // parameter 0
	func->SetParameter(1,0.); // parameter 1
	func->SetParameter(2,1.); // parameter 2
	func->SetParameter(3,0.); // parameter 3
	func->SetParameter(4,1.); // parameter 4
	func->SetParameter(5,0.); // parameter 5
*/



//	func->SetParNames("norm","power","offset", "norm1","power1","offset1", "norm2","power2","offset2");
//	func->SetParNames("norm","offset", "norm1","offset1", "norm2","offset2");
	func->SetParNames("norm", "norm1", "norm2");
        // make the fit
//	efficiencyCostheta->Fit("efficiencyFit");
	efficiencyCostheta->Fit("efficiencyFit", "0");
	

	// get the results
        TF1 *fitResult = efficiencyCostheta->GetFunction("efficiencyFit");
//	norm = fitResult->GetParameter(0);
//	dnorm = fitResult->GetParError(0);
//	power = (Int_t)fitResult->GetParameter(1);
//	if (power%2 != 0) {
//		cout << "cast to Int_t resulted odd power, fixing it to n --> n+1" << endl;
//		power++;
//	}
//	dpower = (Int_t)fitResult->GetParError(1);
//	offset = fitResult->GetParameter(2);
//	doffset = fitResult->GetParError(2);
	norm = fitResult->GetParameter(0);
	dnorm = fitResult->GetParError(0);
//	offset = fitResult->GetParameter(1);
//	doffset = fitResult->GetParError(1);	


//	norm1 = fitResult->GetParameter(3);
//	dnorm1 = fitResult->GetParError(3);
//	power1 = (Int_t)fitResult->GetParameter(4);
//	if (power1%2 != 0) {
//		cout << "cast to Int_t resulted odd power, fixing it to n --> n+1" << endl;
//		power1++;
//	}
//	dpower1 = (Int_t)fitResult->GetParError(4);
//	offset1 = fitResult->GetParameter(5);
//	doffset1 = fitResult->GetParError(5);
	norm1 = fitResult->GetParameter(1);
	dnorm1 = fitResult->GetParError(1);
//	offset1 = fitResult->GetParameter(3);
//	doffset1 = fitResult->GetParError(3);


//	norm2 = fitResult->GetParameter(6);
//	dnorm2 = fitResult->GetParError(6);
//	power2 = (Int_t)fitResult->GetParameter(7);
//	if (power2%2 != 0) {
//		cout << "cast to Int_t resulted odd power, fixing it to n --> n+1" << endl;
//		power2++;
//	}
//	dpower2 = (Int_t)fitResult->GetParError(7);
//	offset2 = fitResult->GetParameter(8);
//	doffset2 = fitResult->GetParError(8);
	norm2 = fitResult->GetParameter(2);
	dnorm2 = fitResult->GetParError(2);
//	offset2 = fitResult->GetParameter(5);
//	doffset2 = fitResult->GetParError(5);


        chi2dof = fitResult->GetChisquare()/fitResult->GetNDF();
        cout 	<< "norm=" << norm << "+/-" << dnorm << "\t"
//		<< "offset=" << offset << "+/-" << doffset << endl;
		<< endl;

	cout    << "norm1=" << norm1 << "+/-" << dnorm1 << "\t"
//                << "offset1=" << offset1 << "+/-" << doffset1 << endl;
		<< endl;

	cout    << "norm2=" << norm2 << "+/-" << dnorm2 << "\t"
//		<< "offset2=" << offset2 << "+/-" << doffset2 << endl;
		<< endl;

	cout	<< "chi2dof=" << chi2dof << endl;

        stmTmp.clear();
        par0Tmp.clear();
        par1Tmp.clear();
	par2Tmp.clear();
	par3Tmp.clear();
	par4Tmp.clear();
	par5Tmp.clear();
	par6Tmp.clear();
	par7Tmp.clear();
	par8Tmp.clear();
	
        //----------
/*
	stmTmp << norm;  stmTmp >> par0Tmp;
        stmTmp.clear();
	stmTmp << power; stmTmp >> par1Tmp;
	stmTmp.clear();
	stmTmp << offset; stmTmp >> par2Tmp;
	stmTmp.clear();

	stmTmp << norm1;  stmTmp >> par3Tmp;
        stmTmp.clear();
	stmTmp << power1; stmTmp >> par4Tmp;
	stmTmp.clear();
        stmTmp << offset1; stmTmp >> par5Tmp;
        stmTmp.clear();
	
	stmTmp << norm2;  stmTmp >> par6Tmp;
        stmTmp.clear();
	stmTmp << power2; stmTmp >> par7Tmp;
	stmTmp.clear();
        stmTmp << offset2; stmTmp >> par8Tmp;
        stmTmp.clear();
*/
	stmTmp << norm;  stmTmp >> par0Tmp;
        stmTmp.clear();

        stmTmp << norm1;  stmTmp >> par1Tmp;
        stmTmp.clear();

        stmTmp << norm2;  stmTmp >> par2Tmp;
        stmTmp.clear();

/*
	stmTmp << norm;  stmTmp >> par0Tmp;
        stmTmp.clear();
        stmTmp << offset; stmTmp >> par1Tmp;
        stmTmp.clear();

        stmTmp << norm1;  stmTmp >> par2Tmp;
        stmTmp.clear();
        stmTmp << offset1; stmTmp >> par3Tmp;
        stmTmp.clear();

        stmTmp << norm2;  stmTmp >> par4Tmp;
        stmTmp.clear();
        stmTmp << offset2; stmTmp >> par5Tmp;
        stmTmp.clear();
*/

        //----------
//	fitFormula = 	"1 - " + par0Tmp + " * (x-" + par2Tmp + ")^" + par1Tmp +
//			" - " + par3Tmp + " * (x-" + par5Tmp + ")^" + par4Tmp +
//			" - " + par6Tmp + " * (x-" + par8Tmp + ")^" + par7Tmp;


	stmTmp << P1; stmTmp >> par6Tmp;
	stmTmp.clear();
	stmTmp << P2; stmTmp >> par7Tmp;
        stmTmp.clear();
	stmTmp << P3; stmTmp >> par8Tmp;
        stmTmp.clear();
/*
	fitFormula =    "1 - " + par0Tmp + " * (x-" + par1Tmp + ")^" + par6Tmp +
                        " - " + par2Tmp + " * (x-" + par3Tmp + ")^" + par7Tmp +
                        " - " + par4Tmp + " * (x-" + par5Tmp + ")^" + par8Tmp;
*/
	fitFormula =    "1 - " + par0Tmp + " * x^" + par6Tmp +
                        " - " + par1Tmp + " * x^" + par7Tmp +
                        " - " + par2Tmp + " * x^" + par8Tmp;

	cout << "fitFormula = " << fitFormula << endl;
	TF1 *f = new TF1("f1", fitFormula.c_str(), -1.1,1.1);
	f->SetLineColor(kBlack);
	f->SetLineWidth(1);
	f->SetLineStyle(1);
//	f->Draw("SAME");
//	fitResult->Draw("SAME");
	//--------------------------------------------





	// ------ legends -----------------------------------------------------------------------------------------------------
	TLegend *leg = new TLegend(0.2,0.65,0.6,0.85);
	leg->AddEntry(cosTheta_noCuts,"Before #eta and p_{T} cuts","L");
	leg->AddEntry(cosTheta_withCuts,"After #eta and p_{T} cuts","L");


	TLegend *legGraph = new TLegend(0.16,0.71,0.45,0.9);
	legGraph->AddEntry(efficiencyCostheta,"Selection efficiency (from MC samples)","P");
	legGraph->AddEntry(f,"#chi^{2} fit","L");


	// frames
	TH2 *frameHisto = new TH2D("", "", 1,cosThetamin-0.5,cosThetamax+0.5, 1,0.,1.2*cosTheta_noCuts->GetMaximum());
	frameHisto->SetXTitle(xtitle.c_str());
	frameHisto->SetYTitle("Events");

	TH2 *frameGraph = new TH2D("", "", 1,cosThetamin-0.5,cosThetamax+0.5, 1,0.,1.2);
	frameGraph->SetXTitle(xtitle.c_str());
	frameGraph->SetYTitle(ytitle.c_str());



	cout << "\n---finalizing---\n" << endl;

	// draw the histos
	pad1->cd();
	frameHisto->Draw();
	cosTheta_noCuts->Draw("sames");
	cosTheta_withCuts->Draw("sames");
	leg->Draw("sames");

	pad2->cd();
	frameGraph->Draw();
	efficiencyCostheta->Draw("P");
	f->Draw("SAME"); // the Fit() call is plotting the result by default
	legGraph->Draw("sames");

	
	// finalize the graphics
	canv->Update();
	canv->Print(selectionEfficiencyRootFileName.c_str(),"root");
	canv->Update();
	canv->Print(selectionEfficiencyEpsFileName.c_str());
	selectionEfficiencyHistFile->Write();

	// wait so the plots can hold on
	canv->WaitPrimitive();

	// Done.
	return 0;
}

