/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/
//--------------------
#include "pythiaPlugin.h"

using namespace std;
using namespace Moses1;



int main() {
	cout << "Holy Moses..." << endl;
	
	
	cout << "\n\n" << "-------------------Set a user defined process run with Pythia8-------------------" << endl;
	// Pythia generator.
	Pythia pythia;

	//------------------------------------------------------
	// Chosen new PDF set; LHAPDF file name conventions.
	//string pdfSet = "cteq61.LHgrid";
	//string pdfSet = "cteq65.LHgrid";
	//string pdfSet = "cteq65.LHpdf";
	//string pdfSet = "cteq5l.LHgrid";
	string pdfSet = "MRST2001lo.LHgrid";
	//string pdfSet = "cteq6.LHpdf";
	//string pdfSet = "MRST2001E.LHgrid";
	int pdfMember_int = 0;
	string pdfMember_str = "0";
	stringstream s;
	if (pdfSet=="cteq6.LHpdf" || pdfSet=="cteq65.LHpdf" || pdfSet=="cteq65.LHgrid") {
		cout << "pdf set is: " << pdfSet << endl;
		cout << "please chose a pdf member (0->40)...";
		cin >> pdfMember_int;
		s << pdfMember_int;
		s >> pdfMember_str;
		if (pdfSet=="cteq6.LHpdf" && pdfMember_int == 0) { pdfSet = "cteq6m.LHpdf"; }
		s.clear();
	}
	else if (pdfSet=="MRST2001E.LHgrid") {
		cout << "pdf set is: " << pdfSet << endl;
		cout << "please chose a pdf member (0=central, 1->30=errors)...";
		cin >> pdfMember_int;
		s << pdfMember_int;
		s >> pdfMember_str;
		s.clear();
	}
	//------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------------------------------
	int outFermionID = 13; // muon
	int nKKmodes = 100; // number of KK modes

	// A class to generate the process from external matrix element. 
	SigmaProcess* myProcess = new KaluzaKlein(outFermionID, nKKmodes, pdfSet, pdfMember_int);
	
	// Hand pointer to Pythia.
	pythia.setSigmaPtr(myProcess);

	// PDF initialization
	pythia.readString("PDF:useLHAPDF = on");
	pythia.readString("PDF:LHAPDFset = " + pdfSet);
	pythia.readString("PDF:LHAPDFmember = " + pdfMember_str);
	// Allow extrapolation of PDF's beyond x and Q2 boundaries, at own risk.
	// Default behaviour is to freeze PDF's at boundaries.
	pythia.readString("PDF:extrapolateLHAPDF = on");
	// Need to change pT0Ref depending on choice of PDF.
	// One possibility: retune to same <n_charged>.
	//pythia.readString("MultipleInteractions:pT0Ref = 10");

	// Create the Z* / gamma* particles.
	// particle: id="23" name="Z0" spinType="3" chargeType="0" colType="0" m0="91.18760" mWidth="2.49520" mMin="10.00000" mMax="0.00000"
	// id:all = name antiName spinType chargeType colType m0 mWidth mMin mMax tau0
	pythia.readString("100023:all = Zstar void 3 0 0 4000.0 4000.0 2000. 6000. 0.");
	pythia.readString("100023:isResonance = false");

	
	bool isSmeared = false;
        int nIsSmeared = 0;
        cout << "select true(1) or smeared(2)...";
        cin >> nIsSmeared;
        isSmeared = (nIsSmeared==2) ? true : false;	


	// phase space cuts
	string sMmin = "";
	string sMmax = "";
	cout << "choose mHat min phasespace...";
	cin >> sMmin;
	cout << "choose mHat max phasespace...";
        cin >> sMmax;
	//pythia.readString("PhaseSpace:mHatMin = 1000");
	pythia.readString("PhaseSpace:mHatMin = "+sMmin);
	pythia.readString("PhaseSpace:mHatMax = "+sMmax);

	// Optionally only compare cross sections.
	string partonLevelSwitch = "n";
	string ISRswitch = "n";
	string FSRswitch = "n";
	cout << "switch off PartonLevel:all ? (y/n)......";
	cin >> partonLevelSwitch;
	if (partonLevelSwitch == "y") {
		pythia.readString("PartonLevel:all = off");
		//pythia.readString("PartonLevel:ISR = off");
	}
	else if (partonLevelSwitch == "n") {
		cout << "switch off PartonLevel:ISR ? (y/n)......";
		cin >> ISRswitch;
		if (ISRswitch == "y") {
			pythia.readString("PartonLevel:ISR = off");
		}
		else if (ISRswitch == "n") {
			pythia.readString("PartonLevel:ISR = on");
		}

		cout << "switch off PartonLevel:FSR ? (y/n)......";
		cin >> FSRswitch;
		if (FSRswitch == "y") {
			pythia.readString("PartonLevel:FSR = off");
		}
	}
	

	// Initialization for LHC with time-based seed.
	pythia.readString("Random:setSeed = on");
	pythia.readString("Random:seed = 0");
	double LHCCME = 14000.;
	cout << "choose LHC center of mass energy in GeV...";
	cin >> LHCCME; 
	//pythia.init(2212, 2212, 14000.);
	pythia.init(2212, 2212, LHCCME);
	cout << "Initialization for LHC with time-based seed" << endl;

	// List changes.
	pythia.settings.listChanged();
	pythia.particleData.listChanged();
	//------------------------------------------------------------------------------------------------------------------------------------
	
	commonOperations commonOps;

	string dirName = "/";


	//string filePath = commonOps.checkANDsetFilepath("MOSESDATA", "/pythiaKaluzaKlein_muons.dat");
	//ofstream muFile;
	//muFile.open(filePath.c_str());

	string filePath2 = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"eventRecord_qqbar2sgmZKK2mumubar.dat");
	ofstream eventRecord;
	eventRecord.open(filePath2.c_str());

	
	//--------------------- for the full generation -----------------------------------------------

	parameters prm; // create an instance of the parameters class
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
	
	physicalParameters physPrm;
	prm.setParameters(physPrm);	


	//extractFromEventRecord extractions;
	double average=0.;
        double sigma=1.;
        double resolution=0.12;   // 12%
        double pTTrackScale=1.e3; // 1 TeV
        bool   printChanges = false;
        extractFromEventRecord extractions;
	if(isSmeared) {
		extractions.setSmear(average, sigma, resolution, pTTrackScale, printChanges);
	}
	extractions.setCuts(runPrm);
	bool fullRecord = true;
	//extractions.setOutputFile("MOSESDATA", "/KK_extractedParticles.dat", fullRecord);
	//extractions.setOutputFile("MOSESDATA", "/PDFstudy/MRST2001E/500_entriesSamples/KK_extractedParticles_subset" + pdfMember_str + ".dat", fullRecord);
	//extractions.setOutputFile("MOSESDATA", "/PDFstudy/MRST2001E/10k_entriesSamples/KK_extractedParticles_subset" + pdfMember_str + ".dat", fullRecord);
	//extractions.setOutputFile("MOSESDATA", "/PDFstudy/CTEQ6/KK_extractedParticles_subset" + pdfMember_str + ".dat", fullRecord);

	//extractions.setOutputFile("MOSESDATA", dirName+"KK_extractedParticles.dat", fullRecord);


	string str;
	stringstream sstrm;
	string dataFileName;
	int nEvents = 0;
	int sampleSizeSelector = 1;
	cout << "select sample size: 1M(1), L=500fb^-1(2), L=100fb^-1(3), manual(4) OR L=10fb^-1(5)..." << endl;
	cin >> sampleSizeSelector;
	if (sampleSizeSelector < 1  || sampleSizeSelector > 5) {
		cout << "invalid option, set to 4 (default)" << endl;
		sampleSizeSelector = 4;
	}
	else {
		switch(sampleSizeSelector) {
			case 1:
				nEvents = 1000000;
				cout << "sample size is 1M (L>> reference sample)" << endl;
				dataFileName = "KK_true_extractedParticles_2trees_1Msample.root";
				if(isSmeared) dataFileName = "KK_smeared_extractedParticles_2trees_1Msample.root";
				break;
			case 2:
				nEvents = 1980;
				cout << "for L=500[fb^-1], sample size is " << nEvents << endl;
				dataFileName = "KK_true_extractedParticles_2trees_500inversefemtobarnSample.root";
				if(isSmeared) dataFileName = "KK_smeared_extractedParticles_2trees_500inversefemtobarnSample.root";
				break;
			case 3:
				//nEvents = 400; // 1->6 TeV with cuts
				//nEvents = 440; // 1->6 TeV without cuts
				nEvents = 644;   // 0.9->14 TeV without cuts
				cout << "for L=100[fb^-1], sample size is " << nEvents << endl;
				dataFileName = "KK_true_extractedParticles_2trees_100inversefemtobarnSample.root";
				if(isSmeared) dataFileName = "KK_smeared_extractedParticles_2trees_100inversefemtobarnSample.root";
				break;
			case 4:
				cout << "\n\nenter the number of events to loop over (integer>0): ";
				cin >> nEvents;
				sstrm.clear();
				sstrm << nEvents;
				sstrm >> str;
				cout << "manual sample size is " << str << endl;
				dataFileName = "KK_true_extractedParticles_2trees_" + str + "_EventsSample.root";
				if(isSmeared) dataFileName = "KK_smeared_extractedParticles_2trees_" + str + "_EventsSample.root";
				sstrm.clear();
				break;
			case 5:
				if(physPrm.mStar==4000.) {
					nEvents = 8129;
					cout << "for E=7[TeV], 200[GeV]<=mHat<=6[TeV] L=10[fb^-1], sample size is " << nEvents << endl;
					dataFileName = "KK_true_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar4TeV.root";
					if(isSmeared) dataFileName = "KK_smeared_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar4TeV.root";
					break;
				}
				else if(physPrm.mStar==3000.) {
                                        nEvents = 7931;
                                        cout << "for E=7[TeV], 200[GeV]<=mHat<=6[TeV] L=10[fb^-1], sample size is " << nEvents << endl;
                                        dataFileName = "KK_true_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar3TeV.root";
                                        if(isSmeared) dataFileName = "KK_smeared_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar3TeV.root";
                                        break;
                                }
			default:
				exit(-1);
		}
	}
	
	//string dataFileName = "KK_extractedParticles_2trees_1Msample.root";
	//	string dataFileName = "KK_extractedParticles.root";
	extractions.setOutputFile("MOSESDATA", dirName+dataFileName, fullRecord, "testTree");
	
	//---------------------------------------------------------------------------------------------

	
	cout << "\n\n" << "---------------------------Start the run--------------------------" << endl;

	bool write2Tree = true;
	bool writeWithAllPairs = true;

	// run
	int mod = (int)nEvents/10;
	int evt=1;
	//int counter = 1; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	bool passCuts = true;
	while (evt <= nEvents) {
		pythia.next();
		//--------------------- for the full generation -----------------------------------------------
		//passCuts = extractions.registerEvent(pythia.event,13); // register the muons (absId=13) to file
//		passCuts = extractions.registerEvent(pythia.event,13, write2Tree); // register the muons (absId=13) to tree
		passCuts = extractions.registerEvent(pythia.event,13, write2Tree, writeWithAllPairs); // register the muons (absId=13) to tree and all the pairs that didn't pass the cuts
		//---------------------------------------------------------------------------------------------
		//muFile << evt << "\t" << pythia.info.mHat() << "\n";
	//	cout << "ev2t=" << evt << " counter=" << counter << "\r" << flush; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		if (evt%mod==0) { cout << "event: " << evt << endl; }
		if (evt==nEvents) {
			pythia.process.list();
			pythia.event.list(eventRecord);
		}
		if (passCuts) { evt++; }
	//	counter++; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	}
	extractions.finalizeOutput(true);
	eventRecord.close();
	pythia.statistics();
	extractions.printInfo();
	
	// Done.
	return 0;
}

