/********************************************************************************************/
/* an example program to demonstrate (in short description) the building of a cross section */
/********************************************************************************************/
//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;

using namespace Moses1;

int main(int argc, char* argv[]) {
	cout << "Holy Moses..." << endl;
	
	TStyle* plotstyle = (TStyle*)gROOT->GetStyle("Plain");
	plotstyle->cd();

	// declare the application
	TApplication theApp("App", &argc, argv);

	commonOperations commonOps; // set the class for commonOperations call
	// set styles
	commonOps.SetOKStyle();


	cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
	cutsEmulator cuts; // create the cuts emulator instance (input to rapidity)


	cout << "runPrm.sMax       = " << sqrt(runPrm.sMax) << endl;
	cout << "runPrm.invMassCut = " << runPrm.invMassCut << endl;
	

	cout << "\n\n" << "--------------Get ALL particle properties in a table--------------" << endl;

	PDTparticle pdt_prt;
	pdt_prt.setPDTtable("HEPPDTTBL","particle.tbl"); // can also initialize with full path and name

	cout << "\n\n" << "-------Set all involved particle properties into 2 vectors--------" << endl;
	bool print = false;
	// initialize an arbitrary array of id's of the leptons/quarks to be loaded. using "pdtParticle" enum
	// d, u, s, c, b, t, e, mu, tau, nu_e, nu_mu, nu_tau
	int idFermions[] = {PDTDWN,PDTUP,PDTSTR,PDTCHM,PDTBOT,PDTTOP,PDTE,PDTMU,PDTTAU,PDTNUE,PDTNUMU,PDTNUTAU};
	// set a vector to contain all pdtParticle structs of properties
	vector<pdtParticle> PDTfermions;
	// fill all quarks and leptons properties in a single vector
	pdt_prt.getAll(idFermions, sizeof(idFermions)/sizeof(*idFermions), PDTfermions, print);

	// do the same for PDTbosons:
	int idBosons[] = {PDTGLU,PDTGAMMA,PDTZ,PDTWPLUS}; // glu, gamma, Z0, W+
	vector<pdtParticle> PDTbosons; // set the pdtParticle properties structs vector
	pdt_prt.getAll(idBosons, sizeof(idBosons)/sizeof(*idBosons), PDTbosons, print); // fill all PDTbosons properties in a single vector


	cout << "\n\n" << "-------Set some more properties manually into the 2 vectors-------" << endl;
	// add to the previously created vector of PDT particle 2 more properties to each properties struct:
	// 1) weakI3, 2) Nc (Number of possible colors)
	// see: mosesParticle.cxx for more details of how this is actually being done
	vector<customParticle> CUSTOMfermions; // define a vector of custom particles
	// put only the selected pdtParticle structs from the PDTfermions vector in the CUSTOMparticle and add more properties:
	CUSTOMparticle custom_prt; // this is only the class instance
	custom_prt.setCUSTOMparticle(PDTfermions, CUSTOMfermions);
	// from now on, have to use the new vector in order to get the custom properties.
	// the PDT vector also exist and can be used.
	// note that the bosons vector is not modified here


	// create relevant classes instances
	couplings cplng; // set the class for couplings call
	partialWidth wdt; // set the class for partialWidth call
	helicityAmplitude hAmp; // set the class for helicityAmplitude call
	rapidity rap;  // set the class for rapidity call
	lhapdf lha; // set the class for lhapdf call
	newZresonance zPrime("chi");
	bool doSMcouplings = true;
	
	// initialize lha (lhapdf instance)
	string pdfSet = "MRST2001lo.LHgrid";
	//string pdfSet = "cteq5l.LHgrid";
	int subset = 0; // it is usually set to '0'
	lha.setlhapdf("LHAPDFSETS",pdfSet,subset); // can also initialize with full path and name

	customParticle outFermion = CUSTOMfermions[7]; // propagator will decay into a muon muon_bar pair
	// the integral over all decay angles
	// [1] theta: distributing like ~ (1+/-cos(theta))^2
	// [2] phi: distributing uniformly
	double angularIntegral = 2*physPrm.pi * 8/3;
	int nMax_ysegments = 100; // for the integration on the rapidity
	int nKKmodes = 100;
	
	Double_t mHat_min = 1000/*10.*/;
	Double_t mHat_max = 6000.;
	Double_t dmHat = 1.;

	//Double_t mKKpeak_min = 2000.;
	
	string dirName = "/";

	double mStar = physPrm.mStar;
	
	int rangeSelector = 0;
	string sRange = "";
	cout << "select range: full(0) or peak(1)...";
	cin >> rangeSelector;
	if(rangeSelector<0 || rangeSelector>1) {
		cout << "invalid option, set to default: 1" << endl;
		rangeSelector = 1;
	}
	if(rangeSelector==1) {
		sRange = "Peak";
		mHat_min = 2000.;
		mHat_max = 5000.;
	}
	else if(rangeSelector==0) {
		sRange = "";
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		mHat_min = runPrm.invMassCut;
		mHat_max = 7000.;
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	}
	Int_t nentries = (Int_t)((mHat_max-mHat_min)/dmHat) + 1;
	cout << "nentries=" << nentries << endl;

	bool bDoCuts = false;
	int nDoCuts = 0;
	cout << "select cuts: without(0) or with(1)...";
	cin >> nDoCuts;
	if (nDoCuts==1) {bDoCuts = true;}
	

	string rootFileName = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"KKvsZE6vsSM_sigma"+sRange+"Total_theory.root");
	string epsFileName = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"KKvsZE6vsSM_sigma"+sRange+"Total_theory.eps");
	TFile *rootFile = new TFile(rootFileName.c_str(),"RECREATE"); // new root file for the cross section + rapidity graph
	TCanvas *c1 = new TCanvas("c1","",0,0,1400,800);
        c1->SetLogy();


	Double_t dCME = 7000.; // 100 GeV step
	Double_t CME_min = 7e3; // 7e3
	Double_t CME_max = 14e3;
	Int_t npoints = (Int_t)((CME_max-CME_min)/dCME) + 1;
	cout << "step size in LHC cm energy is " << dCME << " [GeV], with " << npoints << " steps (from " << CME_min << " to " << CME_max << ")." << endl;

	// alocate arrays for the graph:
	Double_t *CMEnergy = (Double_t *)malloc(npoints*sizeof(Double_t));
	Double_t *totalSigma_SM = (Double_t *)malloc(npoints*sizeof(Double_t));
	Double_t *totalSigma_KK = (Double_t *)malloc(npoints*sizeof(Double_t));
	Double_t *totalSigma_E6 = (Double_t *)malloc(npoints*sizeof(Double_t));
	//--------------------------------------------------------------------//

Double_t CME = CME_min;
for(Int_t k=0 ; k<npoints ; k++)
{
	//----------------------------------//
	/*****/ runPrm.sMax = CME*CME; /*****/
	//----------------------------------//
	
	// for the runs
	double mHat = (double)mHat_min;
        double s_hat = (double)(mHat_min*mHat_min);

	dcomplex fullAmp_SM (0,0);
	dcomplex fullAmp_KK (0,0);
	dcomplex fullAmp_Zprime (0,0);
	double differentialSquareAmp_SM = 0;
	double differentialSquareAmp_KK = 0;
	double differentialSquareAmp_Zprime = 0;
	double squareAmp_SM = 0;
	double squareAmp_KK = 0;
	double squareAmp_Zprime = 0;
	double totalwidth_ZKK = 0;
	double totalwidth_Z0 = 0;
	double totalwidth_Zprime = 0;
	double totalwidth_gammaKK = 0;
	double qqbarRapidity_shat = 0; // this is the rapidity integral factor (single integral)
	

	
	// alocate arrays for the graph:
	Double_t *Sqrt_s_hat = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Sqare_amp_SM = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Sqare_amp_KK = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Sqare_amp_Zprime = (Double_t *)malloc(nentries*sizeof(Double_t));


	
	for (Int_t n=0 ; n<nentries ; n++) { // loop over s_hat
	
		squareAmp_SM = 0; // before each new s_hat it needs to be initialized to '0'
		squareAmp_KK = 0;
		squareAmp_Zprime = 0;
		totalwidth_Z0 = 0; // before each new s_hat it needs to be initialized to '0'
		totalwidth_Zprime = 0;

		// calculate the total width of Z/gamma* for each s_hat:
		// calculate the partial width to this pair of ffbar and summ it to get the total width
		for (int i=0 ; i<(int)CUSTOMfermions.size() ; i++) {
			totalwidth_Z0     += wdt.getpartialWidth(0, s_hat, PDTbosons[2], CUSTOMfermions[i], physPrm); // Z0 total width
			//totalwidth_Zprime += zPrime.getZprimeWidth(doSMcouplings, CUSTOMfermions[i]);
			totalwidth_Zprime += wdt.getpartialWidth(s_hat, PDTbosons[2], CUSTOMfermions[i], physPrm, mStar);
		}

		for (int q=0 ; q<5 ; q++) { // run over all incoming quarks pairs (q+qbar) except for ttbar
			
			differentialSquareAmp_SM = 0; // before each new q it needs to be initialized to '0'
			differentialSquareAmp_KK = 0;
			differentialSquareAmp_Zprime = 0;

			for (double hIn=-0.5 ; hIn<=0.5 ; hIn++) { // loop over incoming helicity states (all quarks)
				for (double hOut=-0.5 ; hOut<=0.5 ; hOut++) { // loop over outgoing helicity states (muon only)

					fullAmp_SM     = dcomplex(0,0); // before each helicity state this has to be initialized to '(0,0)'
					fullAmp_KK     = dcomplex(0,0);
					fullAmp_Zprime = dcomplex(0,0);

					// ---------- SM terms ----------------------------------------------------------------------
					// photon propagator
					fullAmp_SM += hAmp.gethelicityAmplitude(s_hat, 0, hIn, hOut,
										CUSTOMfermions[q], outFermion,
										PDTbosons[1], physPrm);
					// Z0 propagator
					fullAmp_SM += hAmp.gethelicityAmplitude(s_hat, totalwidth_Z0, hIn, hOut,
										CUSTOMfermions[q], outFermion,
										PDTbosons[2], physPrm);
					// ------------------------------------------------------------------------------------------


					// ---------- KK terms ----------------------------------------------------------------------
					fullAmp_KK = fullAmp_SM;
					for (int KKmode=1 ; KKmode<=nKKmodes ; KKmode++) { // loop over all KK modes
						totalwidth_gammaKK = 0.;
						totalwidth_ZKK     = 0.;
						for (int i=0 ; i<(int)CUSTOMfermions.size() ; i++) {
							switch (CUSTOMfermions[i].pdt_prt.id) {
								case PDTNUE: break;
								case PDTNUMU: break;
								case PDTNUTAU: break;
								default: // gamma can not decay into neutrino pair (while Z can)
									totalwidth_gammaKK += wdt.getpartialWidth(KKmode, s_hat, PDTbosons[1],
												CUSTOMfermions[i], physPrm); // gammaKK tot width
									break;
							}
							totalwidth_ZKK    += wdt.getpartialWidth(KKmode, s_hat, PDTbosons[2],
										CUSTOMfermions[i], physPrm); // Z* total width
						}
						
						// photon* propagator
						fullAmp_KK += hAmp.gethelicityAmplitude(KKmode, s_hat, totalwidth_gammaKK, hIn, hOut,
											CUSTOMfermions[q], outFermion,
											PDTbosons[1], physPrm);

						// Z* propagator
						fullAmp_KK += hAmp.gethelicityAmplitude(KKmode, s_hat, totalwidth_ZKK, hIn, hOut,
											CUSTOMfermions[q], outFermion,
											PDTbosons[2], physPrm);
					}
					// ------------------------------------------------------------------------------------------


					// ---------- Z' terms ----------------------------------------------------------------------
					fullAmp_Zprime = zPrime.getSquaredAmplitude(hIn,hOut,s_hat,CUSTOMfermions[q],outFermion,
										totalwidth_Z0,totalwidth_Zprime,doSMcouplings);
					// ------------------------------------------------------------------------------------------


					// get the differential square amplitude: |fullAmp|^2
					differentialSquareAmp_SM     += (s_hat*s_hat) * (1./4.) * real( fullAmp_SM*conj(fullAmp_SM) );
					differentialSquareAmp_KK     += (s_hat*s_hat) * (1./4.) * real( fullAmp_KK*conj(fullAmp_KK) );
					differentialSquareAmp_Zprime += (s_hat*s_hat) * (1./4.) * real( fullAmp_Zprime*conj(fullAmp_Zprime) );
				} // end hOut
			} // end hIn
			
			if (!bDoCuts) {
				qqbarRapidity_shat = rap.simpsonIntegral(nMax_ysegments,s_hat,CUSTOMfermions[q],runPrm,lha);
				// the qqbar rapidity integral factor without cuts
			}
			else {
				qqbarRapidity_shat = rap.simpsonIntegral(nMax_ysegments,s_hat,CUSTOMfermions[q],runPrm,lha,cuts);
				// the qqbar rapidity integral factor with cuts
			}
			// [1] multyply by the rapidity integral factor of the corresponding incoming partons (q+qbar)
			// [2] divide by the number of colors of q since the PDF functions return 9 color combinations
			//     and here only 3 are valid sice the propogator is colorless
			// [3] miltiply by 2 to account the 2 possible origins of the q or the qbar in the beams
			if (!bDoCuts) {
				// no cuts
				differentialSquareAmp_SM     *= 2.*qqbarRapidity_shat/CUSTOMfermions[q].Nc;
				differentialSquareAmp_KK     *= 2.*qqbarRapidity_shat/CUSTOMfermions[q].Nc;
				differentialSquareAmp_Zprime *= 2.*qqbarRapidity_shat/CUSTOMfermions[q].Nc;
			}
			else {
				// with cuts
				// qqbarRapidity_shat already includes the modified integral over theta (because it depend on rapidity)
				differentialSquareAmp_SM     *= 2*(qqbarRapidity_shat/(8./3.))/CUSTOMfermions[q].Nc;
				differentialSquareAmp_KK     *= 2*(qqbarRapidity_shat/(8./3.))/CUSTOMfermions[q].Nc;
				differentialSquareAmp_Zprime *= 2*(qqbarRapidity_shat/(8./3.))/CUSTOMfermions[q].Nc;
			}

			squareAmp_SM     += differentialSquareAmp_SM;  // only then add the differential square amplitude to the full one
			squareAmp_KK     += differentialSquareAmp_KK;
			squareAmp_Zprime += differentialSquareAmp_Zprime;

		} // end q

		// multiply by the coefficients appearing before the squared helicity amplitude
		squareAmp_SM     *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * angularIntegral;
		squareAmp_KK     *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * angularIntegral;
		squareAmp_Zprime *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * angularIntegral;
			// [squareAmp] = [1/GeV^4]
		squareAmp_SM     *= 2*sqrt(s_hat); // dSigma(s_hat) -> dSigma(sqrt(s_hat)) Jacobian of the transformation
		squareAmp_KK     *= 2*sqrt(s_hat);
		squareAmp_Zprime *= 2*sqrt(s_hat);
			// [squareAmp] = [1/GeV^3]
		squareAmp_SM     *= physPrm.GeV2nbarn; // to convert the units to nb/GeV (originally was nb/GeV^2 but here it is nb/GeV)
		squareAmp_KK     *= physPrm.GeV2nbarn;
		squareAmp_Zprime *= physPrm.GeV2nbarn;
			// [squareAmp] = [nb/GeV]
		// see more details about the conversion using the method "prm.printUnits()" in the end of the loop. 

		// fill arrays of graphs:
		Sqare_amp_SM[n]     = squareAmp_SM;
		Sqare_amp_KK[n]     = squareAmp_KK;
		Sqare_amp_Zprime[n] = squareAmp_Zprime;
		Sqrt_s_hat[n] = mHat;


		cout << "sqrt(S)=" << CME << ", sqrt(s_hat)=" << mHat << "\r" << flush;
		mHat += dmHat;
		s_hat = mHat*mHat;

	} // end sMat

        //---------------------------------------------------------------------------------------------------------
	Int_t f1=0, f2=0;
        for(Int_t m=0 ; m<nentries ; m++)
        {
                if (Sqrt_s_hat[m]==1000.) {f1=m;}
                if (Sqrt_s_hat[m]==6000.) {f2=m;}
        }
        if (f2==0) f2 = nentries-1;
        cout << "Sqrt_s_hat[f1=" << f1 << "]=" << Sqrt_s_hat[f1] << endl;
        cout << "Sqrt_s_hat[f2=" << f1 << "]=" << Sqrt_s_hat[f2] << endl;
        double nb2fb = 1.e6;
        double luminosity = 100.; //fb^-1
	if(CME==7000) luminosity = 10.; //fb^-1
        double Nsm = commonOps.simpsonIntegration(Sqare_amp_SM,     Sqrt_s_hat, nentries, f1, f2)*nb2fb*luminosity;
        double Nkk = commonOps.simpsonIntegration(Sqare_amp_KK,     Sqrt_s_hat, nentries, f1, f2)*nb2fb*luminosity;
        double Ne6 = commonOps.simpsonIntegration(Sqare_amp_Zprime, Sqrt_s_hat, nentries, f1, f2)*nb2fb*luminosity;
        cout << "\nNsm=" << Nsm << ", Nkk=" << Nkk << ", Ne6=" << Ne6 << "\n" << endl;
	//---------------------------------------------------------------------------------------------------------
	

        // calculate the integral of the theoretical line (simpson integration method)
        Double_t integrand_SM = 0;
        Double_t integrand_KK = 0;
        Double_t integrand_E6 = 0;

        Double_t sigmaTot_SM = 0;
        Double_t sigmaTot_KK = 0;
        Double_t sigmaTot_E6 = 0;

	
	// find the index of mKKpeak_min in the Sqrt_s_hat and mH_array arrays

/*
	bool stop = false;
	Int_t n=0;
	while (!stop) {
		if (Sqrt_s_hat[n] == mKKpeak_min) {stop = true;}
		n++;
	}
	//cout << "\n" << "starting point: n=" << n << endl;
*/
	Int_t n=0;

	int nSegments = (int)nentries-1;
	int i=0;
	Double_t h = dmHat/3;
        while (i<=nSegments && (i+n)<nentries) {
                // calculate the integrand:
                integrand_SM = Sqare_amp_SM[n+i];
                integrand_KK = Sqare_amp_KK[n+i];
                integrand_E6 = Sqare_amp_Zprime[n+i];
                // simpson:
                if (i==0 || i==nSegments) {
                        sigmaTot_SM += integrand_SM;
                        sigmaTot_KK += integrand_KK;
                        sigmaTot_E6 += integrand_E6;
                }
                else if (i%2==0 && i!=0 && i!=nSegments) {
                        sigmaTot_SM += 2*integrand_SM;
                        sigmaTot_KK += 2*integrand_KK;
                        sigmaTot_E6 += 2*integrand_E6;
                }
		else {
                        sigmaTot_SM += 4*integrand_SM;
                        sigmaTot_KK += 4*integrand_KK;
                        sigmaTot_E6 += 4*integrand_E6;
                }
                // propagate i:
                i++;
        }
        sigmaTot_SM *= h; // [nb]
        sigmaTot_KK *= h; // [nb]
        sigmaTot_E6 *= h; // [nb]

	// fill the array
	CMEnergy[k] = CME;
	totalSigma_SM[k] = sigmaTot_SM;
	totalSigma_E6[k] = sigmaTot_E6;
	totalSigma_KK[k] = sigmaTot_KK;

	if (CME>=13000) {
		cout << "CME=" << CME
		     << " N_SM=" << sigmaTot_SM * 100. * 1e6
		     << " N_E6=" << sigmaTot_E6 * 100. * 1e6
		     << " N_KK=" << sigmaTot_KK * 100. * 1e6
		     << endl;
	}

	if(CME==7000)
	{
		cout << "sigmaTot_SM = " << sigmaTot_SM * 1e6 << " fb, " << "N_SM(10 fb^-1) = " << sigmaTot_SM * 1e6 * 10 << endl;
		cout << "sigmaTot_KK = " << sigmaTot_KK * 1e6 << " fb, " << "N_KK(10 fb^-1) = " << sigmaTot_KK * 1e6 * 10 << endl;
		cout << "sigmaTot_E6 = " << sigmaTot_E6 * 1e6 << " fb, " << "N_E6(10 fb^-1) = " << sigmaTot_E6 * 1e6 * 10 << endl;
	}

	CME += dCME;
}

	

	
	TGraph *crossSection_SM = new TGraph (npoints, CMEnergy, totalSigma_SM);
	TGraph *crossSection_E6 = new TGraph (npoints, CMEnergy, totalSigma_E6);
	TGraph *crossSection_KK = new TGraph (npoints, CMEnergy, totalSigma_KK);


	crossSection_SM->SetTitle("");
	crossSection_SM->SetLineStyle(5);
	crossSection_SM->GetXaxis()->SetTitle("#sqrt{s} [GeV]");
	string sigmaTitle = "";
	if(rangeSelector==0)      { sigmaTitle = "#sigma_{full}#left(#mu^{+}#mu^{-}#right) [nb]"; }
	else if(rangeSelector==1) { sigmaTitle = "#sigma_{peak}#left(#mu^{+}#mu^{-}#right) [nb]"; }
	crossSection_SM->GetYaxis()->SetTitle(sigmaTitle.c_str());
	//crossSection_SM->SetMaximum(1.05*crossSection_KK->GetMaximum());
	crossSection_SM->SetMaximum(1e-5);
	crossSection_SM->Draw("AC");
	
	crossSection_E6->SetLineStyle(3);
	crossSection_E6->Draw("C");

	crossSection_KK->SetLineStyle(1);
	crossSection_KK->Draw("C");

	TLegend *leg = new TLegend(0.2, 0.7, 0.58, 0.9);
	leg->AddEntry(crossSection_SM,"SM","L");
	leg->AddEntry(crossSection_E6,"Z'_{SM}","L");
	leg->AddEntry(crossSection_KK,"KK","L");
	leg->Draw();


	c1->Print(rootFileName.c_str(),"root");
	c1->Update();
	c1->Print(epsFileName.c_str());
	rootFile->Write();

	cout << "\n" << "done..." << endl;

	// wait so the plots can hold on
        c1->WaitPrimitive();

	return 0;
}

