/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/
#include "pythiaPlugin.h"

using namespace std;
using namespace Moses1;



int main() {
	cout << "Holy Moses..." << endl;
	
	
	cout << "\n\n" << "-------------------Set a user defined process run with Pythia8-------------------" << endl;
	// Number of events to generate and to list. Max number of errors.
	// Warning: generation of complete events is much slower than if you use
	// PartonLevel:all = off to only get cross sections, so adjust nEvent.
	
	// Pythia generator.
	Pythia pythia;

	int outFermionID = 13; // muon
	// A class to generate the process from external matrix element. 
	SigmaProcess* myProcess = new eebar2mumubar(outFermionID);
	
	// Hand pointer to Pythia.
	pythia.setSigmaPtr(myProcess);
	cout << "Hand pointer to Pythia" << endl;

	// Allow no substructure in e+- beams: normal for corrected LEP data.
	pythia.readString("PDF:lepton = off");

	// Optionally only compare cross sections.
	string partonLevelSwitch = "n";
	cout << "switch off PartonLevel:all ? (y/n)......";
	cin >> partonLevelSwitch;
	if (partonLevelSwitch == "y") {
		pythia.readString("PartonLevel:all = off");
	}


	// Create the dummy gamma particle so it can be inserted in the event record
	// particle: id="23" name="Z0" spinType="3" chargeType="0" colType="0" m0="91.18760" mWidth="2.49520" mMin="10.00000" mMax="0.00000"
	// id:all = name antiName spinType chargeType colType m0 mWidth mMin mMax tau0

	pythia.readString("10022:all = gammaNoam void 3 0 0 0.00000");

//	pythia.readString("10022:all = gammaNoam void 3 0 0 0.00000 0.00000 10.00000 0.00000");
//	particle: id="22" name="gamma" spinType="3" chargeType="0" colType="0" m0="0.00000"

	pythia.readString("10022:isResonance = false");


	//with time-based seed.
	pythia.readString("Random:setSeed = on");
	pythia.readString("Random:seed = 0");

	pythia.readString("SigmaProcess:alphaEMorder = 1");

	// Initialization for low energies below the Z0 threshold.
	pythia.init(11, -11, 20.);
	cout << "Initialization for LEP" << endl;

	// phase space cuts
	pythia.readString("PhaseSpace:mHatMin = 1");
	pythia.readString("PhaseSpace:mHatMax = 20");

	// List changes.
	pythia.settings.listChanged();
	pythia.particleData.listChanged();

	

	commonOperations commonOps;
	string filePath = commonOps.checkANDsetFilepath("MOSESDATA", "/pythia_external_eebar2mumubar.dat");
	string filePath2 = commonOps.checkANDsetFilepath("MOSESDATA", "/eventRecord_eebar2mumubar.dat");
	ofstream muFile, eventRecord;
	muFile.open(filePath.c_str());
	eventRecord.open(filePath2.c_str());
	
	int nEvents = 200;
	
	cout << "\n\n" << "---------------------------Start the run--------------------------" << endl;
	cout << "\n\nenter the number of MUON-PAIRS to loop over (integer>0, default is 200): ";
	cin >> nEvents;
	cout << "start running and booking events to files..." << endl;

	// run
	int mod = (int)nEvents/10;
	int evt=1;
	while (evt <= nEvents) {
		pythia.next();
		muFile << evt << "\t" << pythia.info.mHat() << "\n";
		if (evt%mod==0) {cout << "event: " << evt << endl;}
		if (evt==nEvents) {
                        pythia.process.list();
                        pythia.event.list(eventRecord);
                }
		evt++;
	}

	cout << "\nfiles were written to:" << endl;
	cout << "\t" << filePath << endl;
	cout << "\t" << filePath2 << endl;
	cout << "\n";
	muFile.close();
	eventRecord.close();
	pythia.statistics();
	
	// Done.
	return 0;
}
