#ifndef NEWZRESONANCE_h
#define NEWZRESONANCE_h
/************************************************************/
/* class to evaluate the user defined helicity amplitude    */
/* the function here is taken from the "Z LINE SHAPE" paper */
/************************************************************/
//----------------
#include "Moses.h"
//----------------
#include "parameters.h"
#include "mosesParticle.h"
#include "couplings.h"


using namespace std;

namespace Moses1 {

	class newZresonance {

	public:
		newZresonance(string model); // chi, psi or eta
		
		virtual ~newZresonance();

		virtual void setBosonPropagators();

		virtual double getVAcouplings(customParticle &fermion, pdtParticle &propagator ,bool isLeft);
	
		virtual double getHelicityCouplings(double helicity, pdtParticle &propagator, customParticle &fermion);

		virtual double getAxialCouplings(pdtParticle &propagator, customParticle &fermion);
		virtual double getVectorCouplings(pdtParticle &propagator, customParticle &fermion);

		virtual double getZprimeWidth(bool doSMcouplings, customParticle &fermion);

		virtual dcomplex getSquaredAmplitude(double hIn, double hOut, double s,
						customParticle &inFermion, customParticle &outFermion,
						double totalZ0width, double totalwidth_Zprime, bool doSMcouplings);
		virtual dcomplex getSquaredAmplitude(double hIn, double hOut, double s, double mZprime,
						customParticle &inFermion, customParticle &outFermion,
						double totalZ0width, double totalwidth_Zprime, bool doSMcouplings);
	private:
		bool m_print;
	
		int m_model;
		double m_theta;
	
		PDTparticle m_pdtPrt; 						    // class
		pdtParticle m_photonPropagator, m_Z0Propagator, m_ZprimePropagator; // struct

		parameters         m_prm;     // class
		physicalParameters m_physPrm; // struct
		runParameters      m_runPrm;  //struct
	
		couplings m_couplings;

		double m_widthMassRatio;

		double m_g2;

		vector< dcomplex > m_frac;
		vector<pdtParticle> m_propagators;
};

}
#endif
