/*****************************************************************/
/* an example program to demonstrate the use of LHAPDF interface */
/* including an example of ROOT analysis of the results          */
/*****************************************************************/

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------


using namespace std;
using namespace Moses1;

int main() {

	cout << "Holy Moses...\n" << endl;

	TStyle* stl = (TStyle*)gROOT->GetStyle("Plain");
	stl->cd();
	
	cout << "\n" << "------------------------Set Bjorken 'x' and scale 'q'------------------------" << endl;
	double x = 0.95; // select Bjorken 'x'
	double q = 10.0; // select scale
	cout << "Bjorken 'x' = " << x << "\t" << "scale 'q' = " << q << endl;



	cout << "\n" << "------------------------print flavor key from 'lhaParton' enum------------------------" << endl;
	cout << "LHADWN=" << LHADWN << "\t" << "LHADWNBAR=" << LHADWNBAR << endl;
	cout << "LHAUP=" << LHAUP << "\t" << "LHAUPBAR=" << LHAUPBAR << endl;
	cout << "LHASTR=" << LHASTR << "\t" << "LHASTRBAR=" << LHASTRBAR << endl;
	cout << "LHACHM=" << LHACHM << "\t" << "LHACHMBAR=" << LHACHMBAR << endl;
	cout << "LHABOT=" << LHABOT << "\t" << "LHABOTBAR=" << LHABOTBAR << endl;
	cout << "LHATOP=" << LHATOP << "\t" << "LHATOPBAR=" << LHATOPBAR << endl;
	cout << "LHAGLU=" << LHAGLU << endl;





	cout << "\n" << "------------------------initialize LHAPDF------------------------" << endl;
	lhapdf lha; // set the class for LHAPDF call
	int f; // specify the flavor of the x*pdf(x) to be evaluated

	int subset = 0; // it is usually set to '0'
	double xpdfx; // a double to put a single flavor x*pdf(x) value in
	vector<double> allxpdfx; // a vector to put all flavors x*pdf(x) values in
	string key; // a string to put a single flavor name in
	vector<string> allkeys; // a vector to put all flavors names in
	//index: -6    -5    -4    -3    -2    -1    0   1 2 3 4 5 6
	//parton: tbar  bbar  cbar  sbar  ubar  dbar glu d u s c b t	




	cout << "\n" << "------------------------call all LHAPDF------------------------" << endl;
	cout << "Demonstrate LHA_PDFs for an all partons call..." << endl;
	cout << "for: flavor = all" << " x=" << x << " q=" << q << endl;

	lha.setlhapdf("LHAPDFSETS","cteq6ll.LHpdf",subset); // can also initialize with full path and name (overload)
	// lha.setlhapdf("/full/path/to/pdfsetfile",subset);
	
	lha.getlhapdf(x,q,allxpdfx);
	lha.prntlhapdf(allxpdfx);
	lha.keylhapdf(allkeys,false); // it can be printed manually from the vector "allkeys". see next lines:
	cout << "----" << endl;
	for (int i=0 ; i<(int)allkeys.size() ; i++) cout << allkeys[i] << endl;
	cout << "----" << endl;
	cout << endl;
	



	cout << "\n" << "------------------------call single LHAPDF------------------------" << endl;
	f = LHATOPBAR; // tbar 
	cout << "Demonstrate LHA_PDFs for a single parton call..." << endl;
	cout << "for: flavor=" << f << " x=" << x << " q=" << q << endl;
	
	lha.getlhapdf(x,q,f,xpdfx);
	lha.prntlhapdf(xpdfx);
	lha.keylhapdf(f,key);
	cout << "---" << endl;
	cout << key << endl;
	cout << "---" << endl;
	cout << endl;


	
	
	cout << "\n" << "------------------------run over 'x' and put all LHAPDF in root file------------------------" << endl;
	cout << "create a root tree for lhapdf's..." << endl;
	//gROOT->Reset();
	Double_t xBj;
	Double_t dxBj;
	Double_t Q = 1e1;
	Double_t glu,u,ubar,d,dbar,s,sbar,c,cbar,b,bbar,t,tbar;

	
	commonOperations commonOps;
	string tree = commonOps.checkANDsetFilepath("MOSESDATA","/pdfTree.root");

	TFile *treeFile = new TFile(tree.c_str(),"RECREATE"); // create a new ROOT file for the pdfTree
	TTree *pdfTree = new TTree("pdfTree","PDF"); // create a Tree to store the PDF's and Bjorken x

	// create the branches:
	pdfTree->Branch("xBj",&xBj,"xBj/D");
	pdfTree->Branch("tbar",&tbar,"tbar/D");
	pdfTree->Branch("bbar",&bbar,"bbar/D");
	pdfTree->Branch("cbar",&cbar,"cbar/D");
	pdfTree->Branch("sbar",&sbar,"sbar/D");
	pdfTree->Branch("ubar",&ubar,"ubar/D");
	pdfTree->Branch("dbar",&dbar,"dbar/D");
	pdfTree->Branch("glu",&glu,"glu/D");
	pdfTree->Branch("d",&d,"d/D");
	pdfTree->Branch("u",&u,"u/D");
	pdfTree->Branch("s",&s,"s/D");
	pdfTree->Branch("c",&c,"c/D");
	pdfTree->Branch("b",&b,"b/D");
	pdfTree->Branch("t",&t,"t/D");
	cout << "Set tree...OK" << endl;

	
	// initialize pdf classes:
	lhapdf lhatest;
	lhatest.setlhapdf("LHAPDFSETS","MRST2006nnlo.LHgrid",subset); // can also initialize with full path and name (overload)
	// lha.setlhapdf("/full/path/to/pdfsetfile",subset);
	// "/usr/local/lhapdf-5.3.1p/share/lhapdf/PDFsets/MRST2006nnlo.LHgrid"
	
	lhatest.getlhapdf(x,q,allxpdfx);
	cout << "Initialize lhapdf...OK" << endl;




	// to check if integrate(sum(x*pdf(x))*dx) = 1
	double test = lhatest.simpsonTest(10000,91.2);//
	cout << "\n\nIntegrate over all x*PDF(s,x) in the range x=[0,1]: " << test << "\t with: " << abs(1-test)*100 << "% deviation\n\n" << endl;




	// alocate arrays
	Int_t nentries = 1000;
	Double_t *Bjorken_x = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *pdftbar = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *pdfbbar = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *pdfcbar = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *pdfsbar = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *pdfubar = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *pdfdbar = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *pdfglu = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *pdfd = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *pdfu = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *pdfs = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *pdfc = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *pdfb = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *pdft = (Double_t *)malloc(nentries*sizeof(Double_t));
	cout << "Allocate arrays...OK" << endl;

	// run ove xBj:
	xBj=1e-7;
	dxBj = (1-xBj)/nentries;
	Int_t i=0;
	char buffer[1000];
	ofstream datafile;

	string bjorken = commonOps.checkANDsetFilepath("MOSESDATA","/PDFvsBjorken_x.dat");
	datafile.open(bjorken.c_str());
	while (i<nentries) {
		lhatest.getlhapdf(xBj,Q,allxpdfx); // get lhapdf's
		tbar = allxpdfx[0];
		bbar = allxpdfx[1];
		cbar = allxpdfx[2];
		sbar = allxpdfx[3];
		ubar = allxpdfx[4];
		dbar = allxpdfx[5];
		glu = allxpdfx[6];
		d = allxpdfx[7];
		u = allxpdfx[8];
		s = allxpdfx[9];
		c = allxpdfx[10];
		b = allxpdfx[11];
		t = allxpdfx[12];

		if (i%(nentries/10)==0) {cout << "Get pdf " << i << "...OK" << endl;}
		
		Bjorken_x[i] = xBj;
		pdftbar[i] = tbar/xBj;
		pdfbbar[i] = bbar/xBj;
		pdfcbar[i] = cbar/xBj;
		pdfsbar[i] = sbar/xBj;
		pdfubar[i] = ubar/xBj;
		pdfdbar[i] = dbar/xBj;
		pdfglu[i] = glu/xBj;
		pdfd[i] = d/xBj;
		pdfu[i] = u/xBj;
		pdfs[i] = s/xBj;
		pdfc[i] = c/xBj;
		pdfb[i] = b/xBj;
		pdft[i] = t/xBj;

		sprintf(buffer,"%12.4E%12.4E%12.4E%12.4E%12.4E%12.4E%12.4E%12.4E%12.4E%12.4E%12.4E%12.4E%12.4E%12.4E",
			xBj,pdftbar[i],pdfbbar[i],pdfcbar[i],pdfsbar[i],pdfubar[i],pdfdbar[i],pdfglu[i],pdfd[i],pdfu[i],
			pdfs[i],pdfc[i],pdfb[i],pdft[i]);
		
		datafile << buffer << endl; // write buffer to data file
		//if (i%10==0) {cout << buffer << endl;}

		pdfTree->Fill(); // fill the tree
		xBj+=dxBj;
		i++;
	}
	cout << "Exit loop...OK" << endl;
	datafile.close();
	cout << "Write file...OK" << endl;
	treeFile->Write();
	cout << "Write tree...OK" << endl;

	// create the graph and register it to another root file
	string graph = commonOps.checkANDsetFilepath("MOSESDATA","/PDFgraphs.root");
	TFile *graphsFile = new TFile(graph.c_str(),"RECREATE"); // new root file for the graph
	TCanvas *c1 = new TCanvas("c1","LHAPDF",0,0,1400,800);
	c1->SetFillColor(37);
	c1->SetLogy();
	//c1->SetLogx();
	//c1->SetGrid();

	TGraph *graph_tbar = new TGraph (nentries, Bjorken_x, pdftbar);
	TGraph *graph_bbar = new TGraph (nentries, Bjorken_x, pdfbbar);
	TGraph *graph_cbar = new TGraph (nentries, Bjorken_x, pdfcbar);
	TGraph *graph_sbar = new TGraph (nentries, Bjorken_x, pdfsbar);
	TGraph *graph_ubar = new TGraph (nentries, Bjorken_x, pdfubar);
	TGraph *graph_dbar = new TGraph (nentries, Bjorken_x, pdfdbar);
	TGraph *graph_glu = new TGraph (nentries, Bjorken_x, pdfglu);
	TGraph *graph_d = new TGraph (nentries, Bjorken_x, pdfd);
	TGraph *graph_u = new TGraph (nentries, Bjorken_x, pdfu);
	TGraph *graph_s = new TGraph (nentries, Bjorken_x, pdfs);
	TGraph *graph_c = new TGraph (nentries, Bjorken_x, pdfc);
	TGraph *graph_b = new TGraph (nentries, Bjorken_x, pdfb);
	TGraph *graph_t = new TGraph (nentries, Bjorken_x, pdft);
	cout << "Graphs....OK" << endl;
	
	// draw graps on top of each other
	graph_glu->SetTitle("Proton LHAPDFs vs Bjorken x (scale q=10GeV)");
	graph_glu->GetXaxis()->SetTitle("Bjorken x");
	graph_glu->GetYaxis()->SetTitle("PDF");
	//graph_glu->GetXaxis()->CenterTitle();
	//graph_glu->GetYaxis()->CenterTitle();
	graph_glu->SetLineColor(kBlue+10);
	graph_glu->Draw("AC");
	graph_d->SetLineColor(kMagenta);
	graph_d->Draw("C");
	graph_u->SetLineColor(kRed+10);
	graph_u->Draw("C");
	graph_s->SetLineColor(kBlue+10);
	graph_s->Draw("C");
	graph_c->SetLineColor(kGreen+10);
	graph_c->Draw("C");
	graph_b->SetLineColor(kYellow+10);
	graph_b->Draw("C");
	graph_t->SetLineColor(kMagenta+10);
	graph_t->Draw("C");
	graph_ubar->SetLineColor(kYellow);
	graph_ubar->Draw("C");
	graph_dbar->SetLineColor(kRed-10);
	graph_dbar->Draw("C");
	graph_sbar->SetLineColor(kGreen);
	graph_sbar->Draw("C");
	graph_cbar->SetLineColor(kBlue);
	graph_cbar->Draw("C");
	graph_bbar->SetLineColor(kRed);
	graph_bbar->Draw("C");
	graph_tbar->SetLineColor(kBlack);
	graph_tbar->Draw("C");


	TLegend *leg = new TLegend(0.91, 0.70, 0.99, 0.99);
	leg->SetFillColor(20);
	leg->AddEntry(graph_tbar,"tbar","L");
	leg->AddEntry(graph_bbar,"bbar","L");
	leg->AddEntry(graph_cbar,"cbar","L");
	leg->AddEntry(graph_sbar,"sbar","L");
	leg->AddEntry(graph_ubar,"ubar","L");
	leg->AddEntry(graph_dbar,"dbar","L");
	leg->AddEntry(graph_glu,"glu","L");
	leg->AddEntry(graph_d,"d","L");
	leg->AddEntry(graph_u,"u","L");
	leg->AddEntry(graph_s,"s","L");
	leg->AddEntry(graph_c,"c","L");
	leg->AddEntry(graph_b,"b","L");
	leg->AddEntry(graph_t,"t","L");
	leg->Draw();
	
	c1->Print(graph.c_str(),"root"); 
	c1->Update();
	graphsFile->Write();

}
