#ifndef MOSESRUNSUMMARY_h
#define MOSESRUNSUMMARY_h

#include "mosesCommon.h"
#include "mosesJetFinderWrap.h"

using namespace std;
using namespace Pythia8;


namespace Moses1 {

class mosesRunSummary
{
	public:
		mosesRunSummary(TFile* file);
		~mosesRunSummary();
		
		void buildSummaryTree();
		void buildSwSummaryTree();
		void buildClusterJetSummaryTree();
		void buildCellJetSummaryTree();
		void buildFastJetSummaryTree();
		
		void fillSummaryTree(Pythia& pythia, int evts);
		void fillSwSummaryTree(string sLHAPDFset);
		void fillClusterJetSummaryTree(mosesClusterJetWrap* clusterJetWrap);
		void fillCellJetSummaryTree(mosesCellJetWrap* cellJetWrap);
		void fillFastJetSummaryTree(mosesFastJetWrap* fastJetWrap);
		
		void writeSummaryTrees();
		void makeAllTreesFriends();
	
		void countProcesses(Pythia& pythia);
		
	private:
		void fillProcesses();
		void printProcesses();
		
		void   writeStringFromBash(string cmd);
		string readStringFromBash(string cmd);
		
	
		// general objects
		TFile* m_file;
		multimap<int, string>* m_processMap;
		
		// sw
		vector<string>* m_sROOTvs;
		vector<string>* m_sLHAPDFvs;
		vector<string>* m_sFASTJETvs;
		vector<string>* m_sHepPDTvs;
		vector<string>* m_sPYTHIA8vs;
		vector<string>* m_sMOSESvs;
		vector<string>* m_sLHAPDFset;
		vector<string>* m_sUserName;
		vector<string>* m_sDate;
		
		// for the run summary
		TTree* m_runSummaryTree;
		TTree* m_clusterJetSummaryTree;
		TTree* m_cellJetSummaryTree;
		TTree* m_fastJetSummaryTree;
		TTree* m_swSummaryTree;
		vector<int>* m_beams_id;
        vector<double>* m_beams_m0;
        vector<double>* m_beams_p0;
        vector<double>* m_beams_pz;
		double m_eCM;
		int m_nEvents;
		vector<string>* m_sProcessName;
		vector<int>*    m_nProcessCode;
		vector<int>*    m_nProcessCount;
		int m_nTried, m_nSelected, m_nAccepted;
		double m_sigma_mb, m_sigmaErr_mb;
		double m_nMHatMin, m_nMHatMax;
		
		// cluster jet settings:
			vector<string>* m_measure;
			int  m_select_cluster;
			int  m_massSet;
			bool m_precluster;
			bool m_reassign;
		// cluster jet analyzer settings:
			double m_yScale;
			double m_pTscale;
			int    m_nJetMin;
			int    m_nJetMax;
		// cell jet settings:
			double m_etaMax;
			int    m_nEta;
			int    m_nPhi;
			int    m_select_cell;
			int    m_smear;
			double m_resolution;
			double m_upperCut;
			double m_threshold;
		// cell jet analyzer settings:
			double m_eTjetMin;
			double m_coneRadius;
			double m_eTseed;
		//fast jet analyzer settings:
			double m_Rparam;
			fastjet::Strategy m_strat;
			int    m_strategy; // enum fastjet::Strategy
			fastjet::RecombinationScheme m_recSchm;
			int    m_recombScheme; // enum fastjet::RecombinationScheme
			fastjet::JetAlgorithm m_jetAlgo;
			int    m_jetAlgorithm; // fastjet::JetAlgorithm
};


}

#endif
