#ifndef USEHEPPDT_h
#define USEHEPPDT_h
/********************************************************/
/* AN INTERFACE CLASS TO HepPDT	                        */
/* class to get particle properties from pdt table file */
/* all methods are using PDT methods                    */
/********************************************************/
//-------------------------
#include "HepPDT/ParticleID.hh"
#include "HepPDT/ParticleData.hh"
#include "HepPDT/TableBuilder.hh"
#include "HepPDT/ParticleDataTable.hh"
#include "HepPDT/TempParticleData.hh"
#include "HepPDT/SpinState.hh"
//-------------------------
#include "HepPID/ParticleIDTranslations.hh"
#include "HepPID/ParticleIDMethods.hh"
#include "HepPID/ParticleName.hh"
#include "HepPID/Version.hh"
//-------------------------
#include "mosesCommon.h"


using namespace std;
using namespace HepPID;

namespace Moses1 {

struct pdtParticle { // a structure to put all regular pdt particle properties
	int id;
	string name;
	double mass;
	double charge;
	double color;
	double Lspin;
	double Sspin;
	double Jspin;
	double width;
	double lifetime;
};


enum pdtEnum { // *** note that this is not a complete list of pdt's particles ***
	PDTDWN=1, PDTUP=2, PDTSTR=3, PDTCHM=4, PDTBOT=5, PDTTOP=6,
	PDTBOTPRIME=7, PDTTOPPRIME=8,
	PDTE=11, PDTNUE=12, PDTMU=13, PDTNUMU=14, PDTTAU=15, PDTNUTAU=16,
	PDTTAUPRIME=17, PDTNUTAUPRIME=18,
	PDTGLU=21, PDTGAMMA=22, PDTZ=23, PDTWPLUS=24, PDTZPRIME0=32,	
	PDTFIRSTMESON=111, PDTLASTMESON=555,
	PDTFIRSTBARYON=1103, PDTLASTBARYON=5554

};
// PDT key to fundamental particles:
// particle:    d u s c b t b' t' e- nu_e mu nu_mu tau- nu_tau
// id:          1 2 3 4 5 6 7  8  11 12   13 14    15   16
// particle:    g  gamma Z0 W+ Higgs0 Z'0
// id:          21 22    23 24 25     32


class PDTparticle {
	public:

		PDTparticle();
		~PDTparticle();
		
		void setPDTtable(string envPath, string fileName); // initialize with environment variable
		void setPDTtable(const char pdttblFile[]); // initialize with full path and file name (overload)

		void getAll(int id, const char outfile[]);
		void getAll(int id, pdtParticle &prop, bool print);
		void getAll(vector<int> id, vector<pdtParticle> &prop, bool print);
		void getAll(const int *id, int id_length, vector<pdtParticle> &prop, bool print);


		string getName(int id);
		double getMass(int id); // can be of type Mesurment
		double getCharge(int id);
		double getColor(int id);
		double getSpin(int id, string state); // can be of type SpinState
		double getWidth(int id);
		double getLifetime(int id);


		// ID translations methods:
		int herwig2pdt(const int herwigID);
		void herwig2pdt_all(string envPath, string fileName);
		int pdt2herwig(const int pdtID);

		int pythia2pdt(const int pythiaID);
		void pythia2pdt_all(string envPath, string fileName);
		int pdt2pythia(const int pdtID);

		int pdg2pdt(const int pdgID);
		void pdg2pdt_all(string envPath, string fileName);
		int pdt2pdg(const int pdtID);
		

	private:
		HepPDT::ParticleDataTable m_datacol;
		int m_pdtID;
};

}
#endif

