# get the date
DATE		= $(shell date +_Day%d%m%YTime%H%M)

# set local directories
SRCDIR          = $(MOSESSYS)/src
OBJARCDIR       = $(MOSESSYS)/obj/archive
OBJDIR          = $(MOSESSYS)/obj
LIBARCDIR       = $(MOSESSYS)/lib/archive
LIBDIR          = $(MOSESSYS)/lib
INCDIR          = $(MOSESSYS)/include
BINDIR		= $(MOSESSYS)/bin/execs
DATDIR		= $(MOSESSYS)/data
ROOTDICTDIR	= $(MOSESSYS)/rootdict

# root (config script available)
ROOTPRFX	= $(shell root-config --prefix)
ROOTLIBS	= $(shell root-config --libs)
ROOTLIBS	+= -glibs -lMinuit
ROOTINC		= $(shell root-config --cflags )
# ROOTINC	= -I$(ROOTSYS)/include

# lhapdf (config script available)
LHAPDFPRFX	= $(shell lhapdf-config --prefix)
LHAPDFLIBS	= $(shell lhapdf-config --libdir)
LHAPDFINC	= $(shell lhapdf-config --incdir)

#FastJet (config script available)
FASTJETPRFX	= $(shell fastjet-config --prefix)
FASTJETLIBS	= $(shell fastjet-config --libs --shared --plugins)
FASTJETINC	= $(shell fastjet-config --cxxflags --plugins)

# heppdt (no config script)
HEPPDTLIBS	= $(HEPPDTSYS)/lib
HEPPDTINC	= $(HEPPDTSYS)/include

# pythia8 (no config script)
PYTHIA8LIBS	= $(PYTHIA8SYS)/lib
PYTHIA8INC	= $(PYTHIA8SYS)/include

#---------------

CXX     = g++
GDB	= -g
SILENT	= -z

CXXFLAGS := -I. -I$(INCDIR)
CXXFLAGS += -I$(LHAPDFINC)
CXXFLAGS += -I$(HEPPDTINC)
CXXFLAGS += -I$(PYTHIA8INC)
CXXFLAGS += $(FASTJETINC)
CXXFLAGS += $(ROOTINC)
CXXFLAGS += -I$(ROOTDICTDIR)
CXXFLAGS += $(SILENT) $(GDB) -fPIC -Wall -O2

AR = ar
ARFLAGS = -r

RANLIB = ranlib
RM = rm -f

CP = cp -f

# one might want to directly use only the EXTERNAL libs (i.e. without Moses interfaces)
EXTRNLIBS := -L$(LHAPDFLIBS) -lLHAPDF
EXTRNLIBS += -L$(HEPPDTLIBS) -lHepPID -lHepPDT
EXTRNLIBS += -L$(PYTHIA8LIBS) -llhapdfdummy -lpythia8
EXTRNLIBS += $(FASTJETLIBS)
EXTRNLIBS += $(ROOTLIBS)

# one might want to use ALL libs (including the EXTRNLIBS and the INTERFACE ones)
CORELIBS := $(EXTRNLIBS)
CORELIBS += -L$(LIBARCDIR) -lMosesCore
CORELIBS += -L$(ROOTDICTDIR) -lrootdict
#CORELIBS += -L$(LIBARCDIR) -lMosesCore -L/srv01/tau/hod/test -lkeyval

LIBFLAGS = -shared
LDFLAGS =

.SUFFIXES : .cxx .cpp .cc .C
